"""유효성 검사를 위한 함수들을 모아놓은 모듈입니다."""

from typing import Union, Tuple

from .customerror import InvalidTypeError


def validate_type(
    allowed_types: Union[Tuple, object],
    *args,
    disallow_none: bool = False,
    exception_type=InvalidTypeError,
):
    """특정 타입에 대해 유효성 검사를 하는 함수입니다.

    허용할 타입을 allowed_types로 받아서, args에 대해 검사합니다.
    allowed_types는 튜플 형태로 여러 타입을 받을 수 있습니다.
    튜플이 아닌 단일 타입을 받을 수도 있습니다.
    disallow_none이 True일 경우 None이 들어오면 예외를 발생시킵니다.
    exception_type은 예외를 발생시킬 때 사용할 예외 타입을 지정합니다.

    Args:
        allowed_types (Union[tuple, object]): 허용할 타입
        args: 검사할 값들
        disallow_none (bool): None을 허용할지 여부
        exception_type : 예외 타입
    """
    if not isinstance(allowed_types, tuple):
        allowed_types = (allowed_types,)
    for value in args:
        if disallow_none and value is None:
            raise exception_type("None이어서는 안됩니다.")

        if value is not None and not isinstance(value, allowed_types):
            raise exception_type(f"{value}는 {allowed_types} 중 하나여야 합니다.")


def validate_str(*args, disallow_none: bool = False):
    """여러 인자에 대해 문자열인지 확인하는 함수입니다.

    validate_type 함수의 allowed_type을 str로 호출하는 함수로, 문자열인지 확인합니다.

    Args:
        args: 검사할 값들
        disallow_none (bool): None을 허용할지 여부
    """
    validate_type(str, *args, disallow_none=disallow_none)


def validate_int(*args, disallow_none: bool = False):
    """여러 인자에 대해 정수형인지 확인하는 함수입니다.

    validate_type 함수의 allowed_type을 int로 호출하는 함수로, 정수형인지 확인합니다.

    Args:
        args: 검사할 값들
        disallow_none (bool): None을 허용할지 여부
    """
    validate_type(int, *args, disallow_none=disallow_none)



"""사용자 정의 예외를 처리하는 모듈입니다.

classes:
    - InvalidTypeError: 유효하지 않은 타입에 대한 예외를 처리하는 클래스
    - InvalidLinkError: Link 형식에 대한 예외를 처리하는 클래스
    - InvalidActionError: 유효하지 않은 action에 대한 예외를 처리하는 클래스
    - InvalidPayloadError: 유효하지 않은 payload에 대한 예외를 처리하는 클래스
"""


class InvalidTypeError(ValueError):
    """유효하지 않은 타입에 대한 예외를 처리하는 클래스입니다."""

    def __init__(self, message: str):
        """InvalidTypeError 인스턴스를 초기화합니다.

        Args:
            message (str): 오류를 설명하는 메시지.
        """
        super().__init__(message)


class InvalidLinkError(ValueError):
    """Link 형식에 대한 예외를 처리하는 클래스입니다."""

    def __init__(self, message: str):
        """InvalidLinkError 인스턴스를 초기화합니다.

        Args:
            message (str): 오류를 설명하는 메시지.
        """
        super().__init__(message)


class InvalidActionError(ValueError):
    """유효하지 않은 action에 대한 예외를 처리하는 클래스입니다."""

    def __init__(self, message: str):
        """InvalidActionError 인스턴스를 초기화합니다.

        Args:
            message (str): 오류를 설명하는 메시지.
        """
        super().__init__(message)


class InvalidPayloadError(ValueError):
    """유효하지 않은 payload에 대한 예외를 처리하는 클래스입니다."""

    def __init__(self, message: str):
        """InvalidPayloadError 인스턴스를 초기화합니다.

        Args:
            message (str): 오류를 설명하는 메시지.
        """
        super().__init__(message)



"""컨텍스트 정보를 담는 클래스를 정의하는 모듈입니다.

컨텍스틑 정보는 Payload 및 Response에서 동일한 형태의 데이터로 사용되기 떄문에,
이를 통일된 형태로 관리하기 위해 별도의 모듈 및 클래스로 정의합니다.
"""

from typing import Optional, Dict
from .base import ParentPayload, SkillTemplate
from .validation import validate_int, validate_str, validate_type


class Context(ParentPayload, SkillTemplate):
    """컨텍스트 정보를 담는 클래스입니다.

    컨텍스트 정보는 봇과 사용자간의 문맥적 상황 공유를 위해 사용됩니다.
    컨텍스트 정보는 이름, 남은 횟수, 유지 시간, 추가 정보를 가집니다.
    챗봇 관리자센터에서 컨텍스트를 설정하지 않으면 이 정보는 무시됩니다.

    이 클래스는 Payload와 Response에서 사용될 수 있도록 공통된 형태로 정의됩니다.
    따라서 from_dict와 render 메서드를 함께 제공합니다.

    Args:
        name (str): 컨텍스트의 이름
        lifespan (int): 컨텍스트의 남은 횟수
        ttl (int, optional): 컨텍스트가 유지되는 시간
        params (Dict, optional): 컨텍스트의 추가 정보

    Attributes:
        name (str): 컨텍스트의 이름
        lifespan (int): 컨텍스트의 남은 횟수
        ttl (int): 컨텍스트가 유지되는 시간
        params (Dict[str, ContextParam]): 컨텍스트의 추가 정보

    Examples:
        >>> context = Context(
        ...     name="context_name",
        ...     lifespan=1,
        ...     ttl=600,
        ...     params={
        ...         "key": {
        ...             "value": "value",
        ...             "resolved_value": "resolved_value"
        ...         }
        ...     }
        ... )
        >>> context.render()
        {
            "name": "context_name",
            "lifespan": 1,
            "ttl": 600,
            "params": {
                "key": "value"
            }
        }
    """

    def __init__(
        self,
        name: str,
        lifespan: int,
        ttl: Optional[int] = None,
        params: Optional[Dict] = None,
    ):
        """Context 객체를 초기화합니다."""
        super().__init__()
        self.name = name
        self.lifespan = lifespan
        self.ttl = ttl
        self.params: Dict[str, ContextParam] = params or {}

    @classmethod
    def from_dict(cls, data: Dict) -> "Context":
        """딕셔너리를 Context 객체로 변환합니다.

        변환할 딕셔너리는 다음과 같은 형태입니다.

        Examples:
            >>> data = {
            ...     "name": "context_name",
            ...     "lifespan": 1,
            ...     "ttl": 600,
            ...     "params": {
            ...         "key": {
            ...             "value": "value",
            ...             "resolvedValue": "resolved_value"
            ...         }
            ...     }
            ... }

        Args:
            data (dict): 변환할 딕셔너리

        Returns:
            Context: 변환된 Context 객체
        """
        name = data.get("name", "")
        lifespan = data.get("lifespan", "")
        ttl = data.get("ttl", None)
        params = {
            key: ContextParam.from_dict(value)
            for key, value in data.get("params", {}).items()
        }
        return cls(name, lifespan, ttl, params)

    def render(self) -> Dict:
        """Context 객체를 카카오톡 응답 규칙에 맞게 딕셔너리로 변환합니다.

        params에 있는 ContextParam 객체는 render 메서드를 통해 value 값을 반환합니다.
        만약, params에 ContextParam 객체가 아닌 다른 객체가 들어있을 경우, 그대로 반환합니다.
        반환되는 딕셔너리는 다음과 같은 형태입니다.

        Examples:
            >>> self.render()
            ... {
            ...     "name": "context_name",
            ...     "lifespan": 1,
            ...     "ttl": 600,
            ...     "params": {
            ...         "key": "value"
            ...     }
            ... }

        Returns:
            dict: Context 객체를 변환한 딕셔너리
        """
        response = {
            "name": self.name,
            "lifeSpan": self.lifespan,
            "ttl": self.ttl,
            "params": {
                key: param.render() if isinstance(param, ContextParam) else param
                for key, param in self.params.items()
            } if self.params else None
        }
        return self.remove_none_item(response)

    def validate(self):
        """Context 객체의 유효성을 검사합니다.

        Raises:
            InvalidTypeError: name이 str이 아닐 경우 발생합니다.
            InvalidTypeError: lifespan이 int가 아닐 경우 발생합니다.
            InvalidTypeError: ttl이 int가 아닐 경우 발생합니다.
            InvalidTypeError: params가 dict가 아닐 경우 발생합니다.
        """
        validate_str(self.name)
        validate_int(self.lifespan, self.ttl)
        validate_type(dict, self.params)


class ContextParam(ParentPayload, SkillTemplate):
    """컨텍스트의 Param 정보를 담는 클래스입니다.

    컨텍스트의 Param 정보는 컨텍스트 정보에 추가적인 정보를 담을 때 사용됩니다.
    Payload에 담겨지는 컨텍스트 정보의 Params는 Stringfy된 JSON 형태로 전달됩니다.
    이떄, `{key : {"value" : value, "resolved_value" : resolved_value}}` 형태로 전달됩니다.
    이후 `Response`에서는 `[key : value]` 형태로 전달해야합니다.
    이 클래스는 이러한 특성에 맞게, `value`와 `resolved_value`를 가지도록 정의됩니다.
    또한, `render()` 메서드를 통해 `value` 값 만을 반환하도록 정의됩니다.

    Args:
        value (str): 컨텍스트의 Param의 value
        resolved_value (str): 컨텍스트의 Param의 resolved_value

    Attributes:
        value (str): 컨텍스트의 Param의 value
        resolved_value (str): 컨텍스트의 Param의 resolved_value

    Examples:
        >>> context_param = ContextParam(
        ...     value="value_str",
        ...     resolved_value="resolved_value"
        ... )
        >>> context_param.render()
        "value_str"
    """

    def __init__(self, value: str, resolved_value: str):
        """ContextParam 객체를 초기화합니다."""
        self.value = value
        self.resolved_value = resolved_value

    @classmethod
    def from_dict(cls, data: dict) -> "ContextParam":
        """딕셔너리를 ContextParam 객체로 변환합니다.

        변환할 딕셔너리는 다음과 같은 형태입니다.

        Examples:
            >>> data = {
            ...     "value": "value",
            ...     "resolvedValue": "resolved_value"
            ... }

        Args:
            data (dict): 변환할 딕셔너리

        Returns:
            ContextParam: 변환된 ContextParam 객체
        """
        value = data.get("value", "")
        resolved_value = data.get("resolvedValue", "")
        return cls(value, resolved_value)

    def render(self) -> str:  # type: ignore
        """ContextParam 객체의 value를 반환합니다.

        이 패키지의 다른 클래스들과 달리, ContextParam 객체는 value만을 반환해야 합니다.
        이는 ContextParam 객체의 특성에 맞게, value 값만을 반환하기 위함입니다

        Returns:
            str: ContextParam 객체의 value

        Examples:
            >>> context_param = ContextParam(
            ...     value="value_str",
            ...     resolved_value="resolved_value"
            ... )
            >>> context_param.render()
            "value_str"
        """
        self.validate()
        return self.value

    def validate(self):
        """ContextParam 객체의 유효성을 검사합니다.

        Raises:
            InvalidTypeError: value가 str이 아닐 경우 발생합니다.
            InvalidTypeError: resolved_value가 str이 아닐 경우 발생합니다.
        """
        validate_str(self.value, self.resolved_value)



"""카카오톡 응답 규칙에 맞는 응답을 생성하는 모듈입니다.

이 모듈은 카카오톡 응답 규칙에 맞는 응답을 생성하는 클래스와 함수를 제공합니다.

packages:
    components: 카카오톡 응답을 위한 컴포넌트 클래스를 정의합니다.

modules:
    base: 카카오톡 응답을 위한 기본 클래스를 정의합니다.
    interaction: 카카오톡 출력 요소 Button과 QuickReply 및 ListItem의 상위 클래스를 정의합니다.
"""

from .base import KakaoResponse, QuickReply, ValidationResponse
from .interaction import ActionEnum
from . import components


__all__ = [
    "KakaoResponse",
    "QuickReply",
    "ValidationResponse",
    "ActionEnum",
    "components",
]



"""카카오톡 출력 요소 Button과 QuickReply 및 ListItem의 상위 클래스를 정의하는 모듈입니다.

이 모듈은 카카오톡 출력 요소 중 클릭시 Action을 취하는 역할을 하는
Button과 QuickReply 및 ListItem의 상위 클래스 Interaction을 정의합니다.

classes:
    - ActionEnum: 카카오톡 출력 요소의 버튼의 동작을 나타내는 열거형 클래스
    - Interaction: 카카오톡 출력 요소 Button과 QuickReply의 상위 클래스
"""

from enum import Enum
from typing import Any, Optional, Tuple, List, Union, Dict

from ..base import BaseModel
from ..customerror import InvalidActionError
from ..validation import validate_str, validate_type
from ..utils import camel_to_snake

__all__ = ["ActionEnum", "Interaction"]


class ActionEnum(Enum):
    """카카오톡 출력 요소의 버튼의 동작을 나타내는 열거형 클래스입니다.

    각 action 유형에 필요한 필드를 반환하는 uses_fields 속성을 가집니다.

    Button과 QuickReply 및 ListItem의 action 속성에 사용됩니다.

    Returns:
        str: action 유형
    """

    WEBLINK = "webLink"
    MESSAGE = "message"
    PHONE = "phone"
    BLOCK = "block"
    SHARE = "share"
    OPERATOR = "operator"

    @property
    def uses_fields(self) -> Tuple[Optional[List[str]], Optional[List[str]]]:
        """각 action 유형이 사용하는 필드를 반환합니다.

        각 action 유형에 필요한 필드와 선택적 필드를 반환합니다.
        tuple의 첫 번째 요소는 필요한 필드, 두 번째 요소는 선택적 필드입니다.
        필요한 필드와 선택적 필드가 없는 경우 각각 None을 반환합니다.

        Returns:
            tuple[Optional[list[str]], Optional[list[str]]]: 필요한 필드와 선택적 필드
        """
        if self == ActionEnum.WEBLINK:
            return ["webLinkUrl"], None
        elif self == ActionEnum.MESSAGE:
            return ["messageText"], None
        elif self == ActionEnum.PHONE:
            return ["phoneNumber"], None
        elif self == ActionEnum.BLOCK:
            return ["blockId"], ["messageText"]
        elif self == ActionEnum.SHARE:
            return None, None
        elif self == ActionEnum.OPERATOR:
            return None, None
        else:
            return None, None


class Interaction(BaseModel):
    """카카오톡 출력 요소 Button과 QuickReply 및 ListItem의 상위 클래스입니다.

    세 출력 요소는 클릭시 Action을 취합니다. 이 클래스는 이들의 상위 클래스입니다.
    클릭시 취할 Action을 나타내는 action과 함께 전달할 데이터를 나타내는 extra를 가집니다.

    Button과 QuickReply, ListItem의 공통 속성을 정의합니다.
    클래스의 공통 속성인 action, extra를 가집니다.
    클래스의 공통 메서드인 validate와 render를 정의합니다.

    Classe Attributes:
        available_action_enums (list[ActionEnum]): 사용 가능한 Action 열거형의 리스트

    Attributes:
        action Union[str, Action]: 버튼의 동작, 문자열 또는 Action 열거형
        extra (dict, optional): 블록을 호출 시 스킬 서버에 추가로 전달할 데이터
    """

    available_action_enums: List[
        ActionEnum
    ] = []  # Action 열거형의 리스트 오버라이드 필요

    def __init__(
        self, action: Optional[Union[str, ActionEnum]], extra: Optional[Dict] = None
    ):
        """Interaction 클래스의 생성자 메서드입니다.

        action을 Action 열거형으로 변환하여 저장합니다.
        action이 None인 경우 None을 저장합니다.

        Args:
            action (Union[str, Action]): 버튼의 동작, 문자열 또는 Action 열거형
            extra (dict, optional): 블록을 호출 시 스킬 서버에 추가로 전달할 데이터
        """
        super().__init__()
        self.action = None
        if action is not None:
            self.action = self.process_action(action)
        self.extra = extra

    @staticmethod
    def process_action(action: Union[str, ActionEnum]) -> ActionEnum:
        """문자열 또는 Action 열거형 인스턴스를 Action 열거형으로 변환합니다.

        action이 문자열인 경우 대문자로 변환하여 Action 열거형에 있는지 확인합니다.
        Action 열거형에 없는 경우 InvalidActionError를 발생시킵니다.

        Args:
            action (Union[str, Action]): QuickReply의 action

        Returns:
            Action: action을 나타내는 Action 열거형

        Raises:
            InvalidActionError: 유효하지 않은 action 값인 경우
        """
        validate_type(
            (str, ActionEnum),
            action,
            disallow_none=True,
            exception_type=InvalidActionError,
        )

        # action이 문자열인 경우 Action 열거형으로 변환
        if isinstance(action, str):
            result = getattr(ActionEnum, action.upper(), None)
            if isinstance(result, ActionEnum):
                action = result
            else:
                raise InvalidActionError(f"유효하지 않은 action 값: {action}")
        return action

    def validate(self):
        """Interaction 객체의 유효성을 검사합니다.

        action이 None인 경우 검사하지 않습니다.
        label이 문자열이 아닌 경우 InvalidTypeError를 발생시킵니다.
        action이 문자열 또는 Action 열거형이 아닌 경우 InvalidActionError를 발생시킵니다.
        이 클래스를 상속받는 클래스에서는 action에 따라 사용하는 필드의 타입을 검사해야 합니다.

        Raises:
            InvalidActionError: action이 문자열 또는 Action 열거형이 아닌 경우
        """
        validate_type(dict, self.extra)

        # action이 None인 경우 검사하지 않음
        if self.action is None:
            return None

        validate_type(
            (str, ActionEnum),
            self.action,
            disallow_none=True,
            exception_type=InvalidActionError,
        )

        # action이 ActionEnum에 있는지 확인
        if self.action not in self.available_action_enums:
            raise InvalidActionError(f"유효하지 않은 action 값: {self.action}")

        # action에 따라 사용하는 필드가 있는경우 validate
        (required_fields, optional_fields) = self.action.uses_fields

        if required_fields is not None:
            for field in required_fields:
                validate_str(getattr(self, camel_to_snake(field)), disallow_none=True)

        if optional_fields is not None:
            for field in optional_fields:
                validate_str(getattr(self, camel_to_snake(field)))

    def render(self) -> Dict:
        """Interaction 객체를 카카오톡 응답 형식에 맞게 딕셔너리로 변환합니다.

        Interaction 객체의 label, action, extra를 딕셔너리로 변환합니다.
        action에 따라 필요한 필드가 있는 경우 반환 값에 포함합니다.

        Returns:
            dict: 카카오톡 응답 형식에 맞게 변환된 Interaction 딕셔너리
        """
        # action이 None인 경우 빈 딕셔너리 반환
        if self.action is None:
            return {}

        self.validate()

        response: Dict[str, Any] = {
            "action": self.action.value,
        }

        # action에 따라 필요한 필드가 있는경우 response에 저장
        (required_fields, optional_fields) = self.action.uses_fields

        if required_fields is not None:
            for camel in required_fields:
                snake = camel_to_snake(camel)
                response[camel] = getattr(self, snake)

        if optional_fields is not None:
            for camel in optional_fields:
                snake = camel_to_snake(camel)
                if getattr(self, snake) is not None:
                    response[camel] = getattr(self, snake)

        if self.extra is not None:
            response["extra"] = self.extra

        return self.remove_none_item(response)



"""카카오톡 간단 출력 요소를 생성하는 클래스를 포함하는 모듈입니다.

이 모듈은 카카오톡 챗봇 응답을 위한 간단한 출력 요소를 생성하는 클래스를 포함합니다.
공통 출력 요소 Common과 카드 출력 요소 Card가 아닌 간단 출력 요소를 포함합니다.

Classes:
    - CarouselComponent: 가로로 출력 요소(ParentComponent)를 나열할 때 사용하는 클래스
    - SimpleTextComponent: 텍스트만을 출력하는 요소를 생성하는 클래스
    - SimpleImageComponent: 이미지만을 출력하는 요소를 생성하는 클래스
"""

from typing import List


from ...validation import validate_str, validate_type
from ..base import ParentComponent
from .card import (
    TextCardComponent,
    BasicCardComponent,
    CommerceCardComponent,
    ItemCardComponent,
    ListCardComponent,
)


class CarouselComponent(ParentComponent):
    """카카오톡 출력 요소 Carousel의 객체를 생성하는 클래스입니다.

    가로로 출력 요소(ParentComponent)를 나열할 때 사용합니다.
    Carousel 내부 객체는 동일한 타입이어야 합니다.

    Attributes:
        items (list[ParentComponent]): CarouselComponent에 포함된 객체 리스트
        type (type[ParentComponent]): CarouselComponent 내부 객체의 타입

    Examples:
        >>> carousel = CarouselComponent()
        >>> carousel.add_item(TextCard(title="제목", description="설명"))
        >>> carousel.add_item(TextCard(title="제목2", description="설명2"))
        >>> carousel.get_dict()
        {
            "version": "2.0",
            "template": {
                "outputs": [
                    {
                        "carousel": {
                            "items": [
                                {
                                    "title": "제목",
                                    "description": "설명"
                                },
                                {
                                    "title": "제목2",
                                    "description": "설명2"
                                }
                            ]
                        }
                    }
                ]
            }
    """

    name = "carousel"

    def __init__(self, *items: ParentComponent):
        """Carousel 객체를 생성합니다.

        items를 받아 객체 리스트를 생성합니다.
        만약 items가 비어있지 않으면 첫 번째 객체의 타입을 type에 저장합니다.
        이후 items에 추가되는 객체는 type과 동일한 타입이어야 합니다.

        Args:
            items (tuple[ParentComponent], optional):
                        CarouselComponent에 포함할 객체 리스트. Defaults to None.
        """
        self.items: List[ParentComponent] = [*items]
        self.type = None
        if not self.is_empty:
            self.type = type(self.items[0])

    @property
    def is_empty(self):
        """CarouselComponent이 비어있는지 확인합니다.

        Returns:
            bool: CarouselComponent이 비어있으면 True, 아니면 False
        """
        return not bool(self.items)

    def add_item(self, item: ParentComponent):
        """CarouselComponent에 객체를 추가합니다.

        만약 CarouselComponent이 비어있으면 객체의 타입을 type에 저장합니다.
        아니면 추가되는 객체가 type과 동일한 타입인지 확인합니다.

        Args:
            item (ParentComponent): 추가할 객체, 아래 클래스 중 하나
                        (TextCardComponent, BasicCardComponent,
                        CommerceCardComponent, ListCardComponent,
                        ItemCardComponent)

        Raises:
            AssertionError: CarouselComponent 내부의 객체는 동일한 타입이 아닌 경우
        """
        if self.is_empty:
            self.type = type(item)
        else:
            assert self.type is not None
            assert isinstance(
                item, self.type
            ), "CarouselComponent 내부의 객체는 동일한 타입이어야 합니다."

        self.items.append(item)

    def remove_item(self, item: ParentComponent):
        """CarouselComponent에서 객체를 제거합니다.

        Args:
            item (ParentComponent): 제거할 객체
        """
        self.items.remove(item)

    def validate(self):
        """객체가 카카오톡 출력 요소에 맞는지 확인합니다.

        Raises:
            AssertionError: CarouselComponent은 최소 1개의 객체를 포함해야 합니다.
            AssertionError: CarouselComponent 내부 객체는 서로 동일한 타입이어야 합니다.
        """
        super().validate()
        assert (
            len(self.items) > 0
        ), "CarouselComponent은 최소 1개의 객체를 포함해야 합니다."
        assert self.type is not None

        if self.type not in (
            TextCardComponent,
            BasicCardComponent,
            CommerceCardComponent,
            ListCardComponent,
            ItemCardComponent,
        ):
            raise AssertionError(
                (
                    "CarouselComponent 내부 객체는 "
                    "TextCardComponent, BasicCardComponent, "
                    "CommerceCardComponent, ListCardComponent, ItemCardComponent "
                    "중 하나여야 합니다."
                )
            )

        validate_type(self.type, *self.items, disallow_none=True)
        for component in self.items:
            component.validate()

    def render(self):
        """객체의 응답 내용을 반환합니다.

        CarouselComponent 객체의 응답 내용을 반환합니다.
        이 응답 내용을 이용하여 render() 메서드에서 최종 응답을 생성합니다.

        Returns:
            dict: 응답 내용

        Examples:
            >>> self.render()
            {
                "type": "carousel",
                "items": [
                    {
                        "title": "제목",
                        "description": "설명"
                    },
                    {
                        "title": "제목2",
                        "description": "설명2"
                    }
                ]
            }
        """
        self.validate()
        assert self.type is not None
        return {
            "type": self.type.name,
            "items": [component.render() for component in self.items],
        }


class SimpleTextComponent(ParentComponent):
    """카카오톡 출력 요소 SimpleText의 객체를 생성하는 클래스입니다.

    SimpleTextComponent는 텍스트 만을 출력하는 요소입니다.

    Attributes:
        text (str): 응답할 텍스트

    Raises:
        ValueError: text가 문자열이 아닌 경우

    Examples:
        >>> simple_text = SimpleTextComponent("안녕하세요")
        >>> simple_text.render()
        {
            "text": "안녕하세요"
        }
    """

    name = "simpleText"

    def __init__(self, text: str):
        """SimpleTextComponent 객체를 생성합니다.

        Args:
            text (str): 응답할 텍스트
        """
        self.text = text

    def validate(self):
        """객체가 카카오톡 출력 요소에 맞는지 확인합니다.

        Raises:
            ValueError: text가 문자열이 아닌 경우
        """
        super().validate()
        return validate_str(self.text)

    def render(self):
        """객체의 응답 내용을 반환합니다.

        SimpleTextComponent 객체의 응답 내용을 반환합니다.
        이 응답 내용을 이용하여 render() 메서드에서 최종 응답을 생성합니다.

        Returns:
            dict: 응답 내용
        """
        self.validate()
        return {"text": self.text}


class SimpleImageComponent(ParentComponent):
    """카카오톡 출력 요소 SimpleImage의 객체를 생성하는 클래스입니다.

    SimpleImageComponent는 이미지 만을 출력하는 요소입니다.

    Attributes:
        image_url (str): 이미지의 URL
        alt_text (str): 대체 텍스트

    Raises:
        ValueError: image_url, alt_text가 문자열이 아닌 경우

    Examples:
        >>> simple_image = SimpleImageComponent(
                image_url="https://example.com/image.jpg",
                alt_text="이미지 설명"
            )
        >>> simple_image.render()
        {
            "imageUrl": "https://example.com/image.jpg",
            "altText": "이미지 설명"
        }
    """

    name = "simpleImage"

    def __init__(self, image_url: str, alt_text: str):
        """SimpleImageComponent 객체를 생성합니다.

        이미지의 URL과 대체 텍스트를 입력받아 객체를 생성합니다.
        이미지는 URL로만 제공되며, 이미지 파일 자체는 전달할 수 없습니다.

        Args:
            image_url (str): 이미지의 URL
            alt_text (str): 대체 텍스트
        """
        super().__init__()
        self.image_url = image_url
        self.alt_text = alt_text

    def validate(self):
        """객체가 카카오톡 출력 요소에 맞는지 확인합니다.

        Raises:
            ValueError: image_url, alt_text가 문자열이 아닌 경우
        """
        super().validate()
        return validate_str(self.image_url, self.alt_text)

    def render(self):
        """객체의 응답 내용을 반환합니다.

        SimpleImageComponent 객체의 응답 내용을 반환합니다.
        이 응답 내용을 이용하여 render() 메서드에서 최종 응답을 생성합니다.

        Returns:
            dict: 응답 내용
        """
        self.validate()
        return {"imageUrl": self.image_url, "altText": self.alt_text}



"""Component의 ItemCard를 구성하는 클래스들을 담고 있는 모듈입니다.

ItemCard는 구성 요소가 다양하며 각 구성 요소는 ItemCard에서만 사용되기 떄문에
Common 모듈에서 독립적인 모듈로 분리하여 ItemCard를 구성하는 클래스들을 담고 있습니다.

classes:
    - ItemThumbnail: ItemCard의 썸네일을 담는 클래스
    - ImageTitle: ItemCard의 이미지 타이틀을 담는 클래스
    - Item: ItemCard의 아이템을 담는 클래스
    - ItemList: ItemCard를 구성하는 Item들을 담는 클래스
    - ItemListSummary: ItemCard의 아이템 리스트 요약을 담는 클래스
    - ItemProfile: ItemCard의 프로필을 담는 클래스
"""

from typing import Optional, Dict

from .common import Common, Link
from ...validation import validate_str, validate_int

__all__ = ["ItemThumbnail", "ImageTitle", "Item", "ItemListSummary", "ItemProfile"]


class ItemThumbnail(Common):
    """ItemCard의 썸네일을 담는 클래스입니다.

    썸네일은 ItemCard의 상단 이미지입니다.

    Attributes:
        image_url (str): 썸네일 이미지 URL
        width (int): 썸네일 이미지의 너비 정보
        height (int): 썸네일 이미지의 높이 정보
        link (Link): 썸네일 이미지 클릭 시 이동할 링크
    """

    def __init__(
        self,
        image_url: str,
        width: Optional[int] = None,
        height: Optional[int] = None,
        link: Optional[Link] = None,
    ):
        """ItemThumbnail을 생성합니다.

        Args:
            image_url (str): 썸네일 이미지 URL
            width (int, optional): 썸네일 이미지의 너비 정보. Defaults to None.
            height (int, optional): 썸네일 이미지의 높이 정보. Defaults to None.
            link (Link, optional): 썸네일 이미지 클릭 시 이동할 링크. Defaults to None.
        """
        super().__init__()
        self.image_url = image_url
        self.width = width
        self.height = height
        self.link = link

    def validate(self):
        """ItemThumbnail의 유효성을 검사합니다.

        Raises:
            AssertionError: image_url이 None이거나 str이 아닐 경우 발생합니다.
            AssertionError: width 또는 height가 int가 아닐 경우 발생합니다.
        """
        validate_str(self.image_url, disallow_none=True)
        validate_int(self.width, self.height)

    def render(self):
        """ItemThumbnail을 렌더링합니다.

        Examples:
            >>> self.render()
            {
            "imageUrl": "https://example.com/1.jpg",
            "width": 640,
            "height": 640,
            "link": {
                "web": "https://example.com/1",
                "mobile": "https://example.com/1"
                }
            }

        Returns:
            dict: 렌더링된 ItemThumbnail
        """
        self.validate()
        response = {
            "imageUrl": self.image_url,
            "width": self.width,
            "height": self.height,
            "link": self.link.render() if self.link is not None else None,
        }
        return self.remove_none_item(response)


class ImageTitle(Common):
    """ItemCard의 이미지 타이틀을 담는 클래스입니다.

    이미지 타이틀은 ItemCard의 상단 이미지 아래에 위치하는 텍스트입니다.
    왼쪽에 텍스트를 표시하고, 이미지를 오른쪽에 표시할 수 있습니다.
    이미지는 필수가 아닙니다.

    Attributes:
        title (str): 이미지 타이틀의 제목
        description (str): 이미지 타이틀의 설명
        image_url (str): 이미지 타이틀의 이미지 URL
    """

    def __init__(
        self,
        title: str,
        description: Optional[str] = None,
        image_url: Optional[str] = None,
    ):
        """ImageTitle을 생성합니다.

        Args:
            title (str): 이미지 타이틀의 제목
            description (str, optional): 이미지 타이틀의 설명. Defaults to None.
            image_url (str, optional): 이미지 타이틀의 이미지 URL. Defaults to None.
        """
        super().__init__()
        self.title = title
        self.description = description
        self.image_url = image_url

    def validate(self):
        """ImageTitle의 유효성을 검사합니다.

        Raises:
            AssertionError: title이 None이거나 str이 아닐 경우 발생합니다.
            AssertionError: description이 str이 아닐 경우 발생합니다.
        """
        validate_str(self.title, disallow_none=True)
        validate_str(self.description)

    def render(self) -> Dict:
        """ImageTitle을 렌더링합니다.

        Returns:
            dict: 렌더링된 ImageTitle

        Examples:
            >>> self.render()
            {
            "title": "제목",
            "description": "설명",
            "imageUrl": "https://example.com/1.jpg"
            }

        """
        self.validate()
        response = {
            "title": self.title,
            "description": self.description,
            "imageUrl": self.image_url,
        }
        return self.remove_none_item(response)


class Item(Common):
    """ItemCard의 아이템을 담는 클래스입니다.

    ItemCard에는 최소 1개 이상의 Item이 있어야 합니다.
    Item은 제목과 설명을 가지고 있습니다.

    Attributes:
        title (str): 아이템의 제목
        description (str): 아이템의 설명
    """

    def __init__(self, title: str, description: str):
        """Item을 생성합니다.

        Args:
            title (str): 아이템의 제목
            description (str): 아이템의 설명
        """
        super().__init__()
        self.title = title
        self.description = description

    def validate(self):
        """Item의 유효성을 검사합니다.

        Raises:
            AssertionError: title이 None이거나 str이 아닐 경우 발생합니다.
        AssertionError: description이 None이거나 str이 아닐 경우 발생합니다.
        """
        validate_str(self.title, self.description, disallow_none=True)

    def render(self) -> Dict:
        """Item을 렌더링합니다.

        Returns:
            dict: 렌더링된 Item

        Examples:
            >>> self.render()
            {
            "title": "제목",
            "description": "설명"
            }
        """
        self.validate()
        response = {"title": self.title, "description": self.description}
        return self.remove_none_item(response)


class ItemListSummary(Item):
    """ItemCard의 아이템 리스트 요약을 담는 클래스입니다.

    Item과 구조가 같지만, ItemListSummary는 아이템 가격 정보를 의미합니다.
    """


class ItemProfile(Common):
    """ItemCard의 프로필을 담는 클래스입니다.

    ItemCard의 프로필은 ItemCard의 썸네일 바로 밑에 위치하는 프로필 정보입니다.

    Attributes:
        title (str): 프로필의 제목
        image_url (str): 프로필 이미지 URL
        width (int): 프로필 이미지의 너비 정보 / 1:1 비율
        height (int): 프로필 이미지의 높이 정보 / 1:1 비율
    """

    def __init__(
        self,
        title: str,
        image_url: Optional[str] = None,
        width: Optional[int] = None,
        height: Optional[int] = None,
    ):
        """ItemProfile을 생성합니다.

        Args:
            title (str): 프로필의 제목
            image_url (str, optional): 프로필 이미지 URL. Defaults to None.
            width (int, optional): 프로필 이미지의 너비 정보. Defaults to None.
            height (int, optional): 프로필 이미지의 높이 정보. Defaults to None.
        """
        super().__init__()
        self.title = title
        self.image_url = image_url
        self.width = width
        self.height = height

    def validate(self):
        """ItemProfile의 유효성을 검사합니다.

        Raises:
            AssertionError: title이 None이거나 str이 아닐 경우 발생합니다.
            AssertionError: image_url이 None이거나 str이 아닐 경우 발생합니다.
            AssertionError: width 또는 height가 int가 아닐 경우 발생합니다.
        """
        validate_str(self.title, disallow_none=True)
        validate_str(self.image_url)
        validate_int(self.width, self.height)

    def render(self) -> Dict:
        """ItemProfile을 렌더링합니다.

        Returns:
            dict: 렌더링된 ItemProfile

        Examples:
            >>> self.render()
            {
                "title": "제목",
                "imageUrl": "https://example.com/1.jpg",
                "width": 640,
                "height": 640
            }
        """
        self.validate()
        response = {
            "title": self.title,
            "imageUrl": self.image_url,
            "width": self.width,
            "height": self.height,
        }
        return self.remove_none_item(response)



"""스킬 실행시 봇 시스템이 스킬 서버에게 전달하는 정보를 객체화한 Payload 객체들을 정의합니다.

classes:
    - ParentPayload: Payload 객체의 부모 클래스
    - Param: openbuilder에서 'detailParams'를 통해 제공되는 파라미터 정보를 저장하는 클래스
    - Params: 블록의 발화에서 추출한 파라미터 정보들을 담는 클래스
    - Action: payload의 'action' 항목을 받아 객체로 변환하는 클래스
    - Bot: payload의 'bot' 항목을 받아 객체로 변환하는 클래스
    - Knowledge: Knowledge 객체를 저장하는 클래스
    - IntentExtra: Intent 객체의 extra 항목을 받아 객체로 변환하는 클래스
    - Intent: payload의 'intent' 항목을 받아 객체로 변환하는 클래스
    - UserProperties: 사용자의 정보를 저장하는 클래스
    - User: Payload의 User 값을 저장하는 클래스
    - UserRequest: Payload의 userRequest 항목을 받아 객체로 변환하는 클래스
    - Payload: Payload 객체를 저장하는 클래스
"""

import json
from enum import Enum
from typing import Optional, Union, List, Dict

from .base import ParentPayload
from .context import Context
from .customerror import InvalidPayloadError


class Param(ParentPayload):
    """Params의 속성으로 사용되는 Param 클래스입니다.

    openbuilder에서 'detailParams'를 통해 제공되는 파라미터 정보를 저장합니다.

    이 클래스는 블록의 발화로부터 추출된 개별 파라미터 정보를 담습니다. openbuilder는 파라미터 정보를
    'param'과 'detailParams'라는 두 가지 key로 구분하여 제공합니다. 이 중 'detailParams'는
    'param'에 포함된 정보와 추가적인 상세 정보를 모두 포함하고 있어, 이 클래스에서는 주로 'detailParams'
    에서 제공되는 정보를 사용합니다.

    파라미터의 값이 딕셔너리 타입인 경우, 해당 딕셔너리는 추가적인 파라미터 상세 정보를 담고 있으며,
    이 정보는 클래스의 속성으로 설정됩니다.

    Attributes:
        origin (str): 사용자 발화에서 직접 추출한 파라미터의 원본 텍스트.
        value Union[str, dict]: 파라미터의 실제 값. 문자열 또는 딕셔너리 형태일 수 있으며,
                            딕셔너리 형태인 경우, 추가적인 상세 정보를 포함합니다.
        group_name (str): 파라미터가 속한 그룹의 이름. 파라미터를 분류하는 데 사용됩니다.
    """

    def __init__(
        self, origin: str, value: Union[str, Dict], group_name: str = "", **kwargs
    ):
        """Param 객체를 생성하는 메서드입니다.

        Args:
            origin (str): 사용자가 입력한 발화에서 추출한 파라미터의 원본
            value (Union[str, dict]): 파라미터의 대표값
            group_name (str): 파라미터의 그룹 이름 (기본값: '')
            **kwargs: 추가적인 정보
        """
        self.origin = origin
        self.value = value
        self.group_name = group_name
        for k, v in kwargs.items():  # 추가적인 정보를 객체의 속성으로 추가
            setattr(self, k, v)

    @classmethod
    def from_dict(cls, data: Dict):
        """딕셔너리에서 Param 객체를 생성하는 클래스 메서드입니다.

        이 메서드는 'detailParams'에서 제공된 파라미터 정보를 바탕으로 Param 객체를 생성합니다.
        제공된 딕셔너리는 파라미터의 'origin', 'value', 그리고 선택적으로 'groupName'을 포함할 수 있습니다.
        'value'가 딕셔너리 형태인 경우, 이는 Param 객체의 추가적인 속성으로 포함됩니다.

        Args:
            data (dict): 파라미터 정보를 포함하는 딕셔너리. 이 딕셔너리는 'origin', 'value',
                        그리고 선택적으로 'groupName' 키를 포함해야 합니다.

        Returns:
            Param: 생성된 Param 객체. 'origin', 'value', 'groupName' (있는 경우)을
                속성으로 가지며, 'value'가 딕셔너리인 경우 추가적인 속성을 동적으로 생성합니다.
        """
        try:
            origin = data.get("origin", "")
            value = data.get("value", "")
            group_name = data.get("groupName", "")
            if isinstance(value, dict):
                # 'value'가 딕셔너리 타입인 경우, 이를 객체의 속성으로 추가합니다.
                return cls(origin=origin, group_name=group_name, **value)
            else:
                return cls(origin=origin, value=value, group_name=group_name)
        except KeyError as err:
            raise InvalidPayloadError(
                "Param 객체를 생성하기 위한 키가 존재하지 않습니다."
            ) from err


class Action(ParentPayload):
    """payload의 'action' 항목을 받아 객체로 변환하는 클래스입니다.

    Payload 객체의 action 속성으로 사용되는 클래스입니다.

    이 클래스는 사용자의 요청에 대한 액션 정보를 저장합니다. 액션 정보에는 액션의 고유 ID,
    액션의 이름, 액션에 포함된 파라미터 정보(Params 객체), 상세 파라미터 정보, 그리고
    클라이언트 추가 정보가 포함됩니다.

    Attributes:
        id (str): 액션의 고유 ID입니다.
        name (str): 액션의 이름입니다.
        params (Params): 액션에 포함된 파라미터 정보를 담는 Params 객체입니다.
        client_extra (dict): 클라이언트에서 추가적으로 제공하는 정보입니다.
    """

    def __init__(
        self,
        ID: str,
        name: str,
        params: Dict[str, str],
        detail_params: Dict[str, Param],
        client_extra: Dict,
    ):
        """Action 클래스의 인스턴스를 초기화합니다.

        사용자의 액션 정보를 초기화하며, 액션의 고유 ID, 이름, 파라미터 정보, 상세 파라미터 정보,
        그리고 클라이언트 추가 정보를 설정합니다.

        Args:
            ID (str): 액션의 고유 ID.
            name (str): 액션의 이름.
            params (Dict[str, str]): 액션에 포함된 파라미터 정보.
            detail_params (Dict[str, Param]): 상세 파라미터 정보.
            client_extra (Dict[str, str]): 클라이언트 추가 정보.
        """
        self.id = ID
        self.name = name
        self.params = params
        self.detail_params = detail_params
        self.client_extra = client_extra

    @classmethod
    def from_dict(cls, data: Dict) -> "Action":
        """딕셔너리에서 Action 객체를 생성하는 클래스 메서드입니다.

        제공해야 하는 딕셔너리는 payload의 'action'입니다.
        주어진 딕셔너리로부터 액션 정보를 추출하여 Action 클래스의 인스턴스를 생성합니다.
        필요한 모든 키가 존재하지 않는 경우, InvalidPayloadError 예외를 발생시킵니다.

        Args:
            data (dict): 액션 정보가 담긴 딕셔너리.

        Returns:
            Action: 생성된 Action 인스턴스.

        Examples:
            >>> self.from_dict({
            ...     'id': 'action_id',
            ...     'name': 'action_name',
            ...     'params': {
            ...         'param_key': 'param_value'
            ...     },
            ...     'detailParams': {
            ...         'detail_param_key': {
            ...             'origin': 'origin_text',
            ...             'value': 'param_value'
            ...         }
            ...     },
            ...     'clientExtra': {
            ...         'key': 'value'
            ...     }
            ... })
            Action('action_id', 'action_name', {'param_key': 'param_value'},
                   {'detail_param_key': Param('origin_text', 'param_value')},
                   {'key': 'value'})
        """
        ID = data.get("id", "")
        name = data.get("name", "")
        params = data.get("params", {})
        detail_params = {
            key: Param.from_dict(value)
            for key, value in data.get("detailParams", {}).items()
        }
        client_extra = data.get("clientExtra", {})
        return cls(
            ID=ID,
            name=name,
            params=params,
            detail_params=detail_params,
            client_extra=client_extra,
        )


class Bot(ParentPayload):
    """payload의 'bot' 항목을 받아 객체로 변환하는 클래스입니다.

    Payload 객체의 bot 속성으로 사용되는 클래스입니다.
    이 클래스는 봇 정보를 저장합니다. 봇 정보에는 봇의 고유 ID와 이름이 포함됩니다.

    Attributes:
        id (str): 봇의 고유 ID입니다.
        name (str): 봇의 이름입니다.
    """

    def __init__(self, ID: str, name: str):
        """Bot 클래스의 인스턴스를 초기화합니다.

        Args:
            ID (str): 봇의 고유 ID.
            name (str): 봇의 이름.
        """
        self.id = ID
        self.name = name

    @classmethod
    def from_dict(cls, data: Dict) -> "Bot":
        """딕셔너리에서 Bot 객체를 생성하는 클래스 메서드입니다.

        제공해야 하는 딕셔너리는 payload의 'bot'입니다.
        주어진 딕셔너리로부터 봇 정보를 추출하여 Bot 클래스의 인스턴스를 생성합니다.
        필요한 모든 키가 존재하지 않는 경우, InvalidPayloadError 예외를 발생시킵니다.

        Args:
            data (dict): 봇 정보가 담긴 딕셔너리.

        Returns:
            Bot: 생성된 Bot 인스턴스.
        """
        ID = data.get("id", "")
        name = data.get("name", "")
        return cls(ID=ID, name=name)


class Knowledge(ParentPayload):
    """Knowledge 객체를 저장하는 클래스입니다.

    IntentExtra의 matched_knowledges 속성으로 사용되는 클래스입니다.
    이 클래스는 지식+의 지식 정보를 저장합니다.
    발화와 일치하는 지식의 정보를 담기 위해 사용됩니다.
    지식 정보에는 지식의 ID, 이름, 설명, 그리고 추가 정보가 포함됩니다.

    Attributes:
        answer (str): 지식의 답변입니다.
        question (str): 지식의 질문입니다.
        categories (list[str]): 지식의 카테고리입니다.
        landing_url (str): 지식 답변에서 링크 더보기 url 입니다.
        image_url (str): 지식 답변에서 썸네일 이미지 url 입니다.
    """

    def __init__(
        self,
        answer: str,
        question: str,
        categories: List[str],
        landing_url: str,
        image_url: str,
    ):
        """Knowledge 클래스의 인스턴스를 초기화합니다."""
        self.answer = answer
        self.question = question
        self.categories = categories
        self.landing_url = landing_url
        self.image_url = image_url

    @classmethod
    def from_dict(cls, data: Dict) -> "Knowledge":
        """딕셔너리에서 Knowledge 객체를 생성하는 클래스 메서드입니다.

        제공해야 하는 딕셔너리는 payload의 'intent'의 'extra'의
        'matched_knowledges' 배열의 요소 입니다.
        주어진 딕셔너리로부터 지식 정보를 추출하여 Knowledge 클래스의 인스턴스를 생성합니다.
        IntentExtra 객체의 'matched_knowledges' 의 배열 속성으로 사용됩니다.

        Args:
            data (dict): 지식 정보가 담긴 딕셔너리.
        """
        answer = data.get("answer", "")
        question = data.get("question", "")
        categories = data.get("categories", [])
        landing_url = data.get("landingUrl", "")
        image_url = data.get("imageUrl", "")
        return cls(
            answer=answer,
            question=question,
            categories=categories,
            landing_url=landing_url,
            image_url=image_url,
        )


class IntentExtra(ParentPayload):
    """Intent 객체의 extra 항목을 받아 객체로 변환하는 클래스입니다.

    이 클래스는 Intent 객체의 extra 속성으로 사용됩니다.
    extra 항목에는 발화에서 지식+와 일치하는 지식의 목록을 담고 있습니다.
    Payload의 'intent' 항목에서 'extra'를 받아 객체로 변환하기 위해 사용됩니다.
    단독으로 사용되지 않습니다.

    Attributes:
        reason (dict): 정보 없음(Kakao에서 정보를 제공하지 않습니다.)
        matched_knowledges (dict): Intent의 지식을 나타내는 딕셔너리입니다.
    """

    def __init__(
        self,
        reason: Optional[dict] = None,
        matched_knowledges: Optional[list[Knowledge]] = None,
    ):
        """IntentExtra 클래스의 인스턴스를 초기화합니다.

        Args:
            reason (dict): 정보 없음(Kakao에서 정보를 제공하지 않습니다.)
            matched_knowledges (dict): Intent의 지식을 나타내는 딕셔너리.
        """
        if reason is None:
            reason = {}
        self.reason = reason
        self.matched_knowledges = matched_knowledges

    @classmethod
    def from_dict(cls, data: Dict) -> "IntentExtra":
        """딕셔너리에서 IntentExtra 객체를 생성하는 클래스 메서드입니다.

        제공해야 하는 딕셔너리는 payload의 'intent'의 'extra'입니다.
        주어진 딕셔너리로부터 IntentExtra 정보를 추출하여 IntentExtra 클래스의 인스턴스를 생성합니다.

        Args:
            data (dict): IntentExtra 정보가 담긴 딕셔너리.
        """
        reason: Dict = data.get("reason", {})
        knowledges: Dict = data.get("matched_knowledges", {})
        matched_knowledges = [
            Knowledge.from_dict(knowledge) for knowledge in knowledges
        ]
        return cls(reason, matched_knowledges)


class Intent(ParentPayload):
    """payload의 'intent' 항목을 받아 객체로 변환하는 클래스입니다.

    Payload 객체의 intent 속성으로 사용되는 클래스입니다.
    발화와 일치하는 블록이나 지식의 정보를 담고 있는 객체입니다.
    발화가 지식+에 일치하는 경우, 일치하는 지식의 목록을 포함합니다.
    extra는 발화에 일치한 지식 목록을 포함하는 IntentExtra 객체로 구성됩니다.
    단독으로 사용되지 않습니다.

    Attributes:
        id (str): Intent의 고유 ID입니다.
        name (str): Intent의 이름입니다.
        extra (IntentExtra): Intent의 추가 정보입니다.
    """

    def __init__(
        self,
        ID: str,
        name: str,
        extra: IntentExtra,
    ):
        """Intent 클래스의 인스턴스를 초기화합니다."""
        self.id = ID
        self.name = name
        self.extra = extra

    @classmethod
    def from_dict(cls, data: Dict) -> "Intent":
        """딕셔너리에서 Intent 객체를 생성하는 클래스 메서드입니다.

        제공해야 하는 딕셔너리는 payload의 'intent'입니다.
        주어진 딕셔너리로부터 Intent 정보를 추출하여 Intent 클래스의 인스턴스를 생성합니다.

        Args:
            data (dict): Intent 정보가 담긴 딕셔너리.

        Returns:
            Intent: 생성된 Intent 인스턴스.
        """
        ID = data.get("id", "")
        name = data.get("name", "")
        extra = IntentExtra.from_dict(data.get("extra", {}))
        return cls(ID=ID, name=name, extra=extra)


class UserProperties(ParentPayload):
    """사용자의 정보를 저장하는 클래스입니다.

    UserRequest 객체의 user 속성으로 사용됩니다.
    추가적으로 제공하는 사용자의 속성 정보를 담고 있습니다.
    단독으로 사용되지 않습니다.

    Attributes:
        plusfriend_user_key (str): 카카오톡 채널에서 제공하는 사용자 식별키
        app_user_id (str): 봇 설정에서 앱키를 설정한 경우에만 제공되는 사용자 정보입니다.
        is_friend (bool): 사용자가 봇과 연결된 카카오톡 채널을 추가한 경우 제공되는 식별키입니다.
    """

    def __init__(
        self,
        plusfriend_user_key: str,
        app_user_id: str,
        is_friend: Optional[bool] = None,
    ):
        """UserProperties 객체를 생성하는 메서드입니다.

        Args:
            plusfriend_user_key (str): 카카오톡 채널에서 제공하는 사용자 식별키
            app_user_id (str): 봇 설정에서 앱키를 설정한 경우에만 제공되는 사용자 정보입니다.
            is_friend (bool): 사용자가 봇과 연결된 카카오톡 채널을 추가한 경우 제공되는 식별키입니다.
        """
        self.plusfriend_user_key = plusfriend_user_key
        self.app_user_id = app_user_id
        self.is_friend = is_friend

    @classmethod
    def from_dict(cls, data: Dict):
        """딕셔너리에서 UserProperties 객체를 생성하는 클래스 메서드입니다.

        전달받는 딕셔너리는 payload의 'user'의 'properties'입니다.
        주어진 딕셔너리로부터 사용자 정보를 추출하여 UserProperties 클래스의 인스턴스를 생성합니다.

        Args:
            data (dict): 사용자 정보가 담긴 딕셔너리
        """
        plusfriend_user_key = data.get("plusfriendUserKey", "")
        app_user_id = data.get("appUserId", "")
        is_friend = data.get("isFriend", None)
        return cls(
            plusfriend_user_key=plusfriend_user_key,
            app_user_id=app_user_id,
            is_friend=is_friend,
        )


class User(ParentPayload):
    """Payload의 User 값을 저장하는 클래스입니다.

    UserRequest 객체의 user 속성으로 사용됩니다.
    단독으로 사용되지 않습니다.

    Attributes:
        id (str): 사용자를 식별할 수 있는 key로 최대 70자의 값
        type (str): 현재는 botUserKey만 제공
        properties (dict): 추가적으로 제공하는 사용자의 속성 정보

    Examples:
    >>> User('user_id', 'botUserKey')
    >>> user.id
    'user_id'
    >>> user.type
    'botUserKey'

    """

    def __init__(
        self,
        ID: str,
        TYPE: str,
        properties: Optional[UserProperties] = None,
    ):
        """User 객체를 생성하는 메서드입니다.

        properties가 None인 경우 빈 딕셔너리로 초기화합니다.
        """
        if properties is None:
            properties = {}

        self.id = ID
        self.type = TYPE
        self.properties = properties

    @classmethod
    def from_dict(cls, data: Dict) -> "User":
        """딕셔너리에서 User 객체를 생성하는 클래스 메서드입니다.

        전달받는 딕셔너리는 payload의 'userRequest'의 'user'입니다.

        Args:
            data (dict): 사용자 정보가 담긴 딕셔너리

        Returns:
            User: 생성된 User 객체
        """
        ID = data.get("id", "")
        TYPE = data.get("type", "")
        properties = UserProperties.from_dict(data.get("properties", {}))
        return cls(ID=ID, TYPE=TYPE, properties=properties)


class UserRequest(ParentPayload):
    """Payload의 userRequest 항목을 받아 객체로 변환하는 클래스입니다.

    Payload 객체의 user_request 속성으로 사용됩니다.
    단독으로 사용되지 않으며, User 객체를 속성으로 가지고 있습니다.

    Attributes:
        timezone (str): 사용자의 시간대
        block (dict): 사용자의 발화에 반응한 블록의 정보 (id, name)
        utterance (str): 사용자 발화
        lang (str): 사용자 발화의 언어, 한국에서 보낸 요청이라면 “ko”
        user (User): 사용자 정보
        params (dict): 정보 없음(Kakao에서 정보를 제공하지 않습니다.)
        callback_url (str): AI 콜백 요청을 전송할 URL

    Examples:
        >>> payload.user_request.timezone
        'Asia/Seoul'
        >>> payload.user_request.block
        {'id': 'block_id', 'name': 'block_name'}
        >>> payload.user_request.utterance
        '사용자 발화'
        >>> payload.user_request.lang
        'ko'
        >>> payload.user_request.user.id
        'user_id'
    """

    def __init__(
        self,
        timezone: str,
        block: Dict,
        utterance: str,
        lang: str,
        user: User,
        params: Optional[Dict] = None,
        callback_url: Optional[str] = None,
    ):
        """UserRequest 객체를 생성하는 메서드입니다."""
        self.timezone = timezone
        self.block = block
        self.utterance = utterance
        self.lang = lang
        self.user = user
        self.params = params
        self.callback_url = callback_url

    @classmethod
    def from_dict(cls, data: Dict):
        """딕셔너리에서 UserRequest 객체를 생성하는 클래스 메서드입니다.

        전달받는 딕셔너리는 payload의 'userRequest'입니다.

        Args:
            data (dict): 사용자 요청 정보가 담긴 딕셔너리
        """
        timezone = data.get("timezone", "")
        block = data.get("block", {})
        utterance = data.get("utterance", "")
        lang = data.get("lang", "")
        user = User.from_dict(data.get("user", {}))
        params = data.get("params")
        callback_url = data.get("callbackUrl")
        return cls(
            timezone=timezone,
            block=block,
            utterance=utterance,
            lang=lang,
            user=user,
            params=params,
            callback_url=callback_url,
        )


class OutputType(str, Enum):
    """카카오 챗봇 트리거의 출력 유형입니다.

    챗봇 구성 요소에서 사용자 입력 또는 버튼/카드 등의 출력 방식에 해당합니다.

    - INPUT: 일반 텍스트 입력
    - CARD_BUTTON: 일반 카드 버튼
    - LIST_ITEM: 리스트 카드 항목
    - LISTMENU: 리스트 메뉴 버튼
    - QUICKREPLY: 바로연결 버튼
    """

    INPUT = "INPUT"
    CARD_BUTTON = "CARD_BUTTON"
    LIST_ITEM = "LIST_ITEM"
    LISTMENU = "LISTMENU"
    QUICKREPLY = "QUICKREPLY"


class ActionType(str, Enum):
    """카카오 챗봇 트리거의 동작 유형입니다.

    버튼 클릭 또는 입력 후 실행되는 동작의 목적에 해당합니다.

    - TEXT: 사용자의 텍스트 입력
    - MESSAGE: 메시지 전송
    - BLOCK: 다른 블록 호출
    """

    TEXT = "TEXT"
    MESSAGE = "MESSAGE"
    BLOCK = "BLOCK"


class TriggerType(str, Enum):
    """사용자 발화를 유발한 트리거의 유형을 정의하는 Enum 클래스입니다.

    TriggerType은 카카오 챗봇의 버튼, 리스트, 퀵리플라이 등의 사용자 인터페이스 요소와 관련된
    입력 유형을 분류하며, 각 트리거는 고유 문자열 값을 갖습니다. 이 클래스는 각 트리거가
    어떤 출력 방식(OutputType)과 동작 방식(ActionType)을 갖는지를 property를 통해 제공합니다.

    이 클래스는 __OUTPUT_TYPE_MAP 및 __ACTION_TYPE_MAP이라는 정적 매핑을 사용하여
    각 Enum 멤버의 output_type과 action_type을 효율적으로 참조합니다.

    Example:
        >>> t = TriggerType.CARD_BUTTON_BLOCK
        >>> print(t.value)
        CARD_BUTTON_BLOCK
        >>> print(t.output_type)
        OutputType.CARD_BUTTON
        >>> print(t.action_type)
        ActionType.BLOCK

        >>> TriggerType("LIST_ITEM_MESSAGE").output_type
        OutputType.LIST_ITEM
        >>> TriggerType("LIST_ITEM_MESSAGE").action_type
        ActionType.MESSAGE

    Raises:
        ValueError: Enum 값이 매핑에 존재하지 않는 경우.
    """

    TEXT_INPUT = "TEXT_INPUT"
    CARD_BUTTON_MESSAGE = "CARD_BUTTON_MESSAGE"
    CARD_BUTTON_BLOCK = "CARD_BUTTON_BLOCK"
    LIST_ITEM_MESSAGE = "LIST_ITEM_MESSAGE"
    LIST_ITEM_BLOCK = "LIST_ITEM_BLOCK"
    LISTMENU_MESSAGE = "LISTMENU_MESSAGE"
    LISTMENU_BLOCK = "LISTMENU_BLOCK"
    QUICKREPLY_BUTTON_MESSAGE = "QUICKREPLY_BUTTON_MESSAGE"
    QUICKREPLY_BUTTON_BLOCK = "QUICKREPLY_BUTTON_BLOCK"

    __OUTPUT_TYPE_MAP: dict[str, OutputType] = {
        "TEXT_INPUT": OutputType.INPUT,
        "CARD_BUTTON_MESSAGE": OutputType.CARD_BUTTON,
        "CARD_BUTTON_BLOCK": OutputType.CARD_BUTTON,
        "LIST_ITEM_MESSAGE": OutputType.LIST_ITEM,
        "LIST_ITEM_BLOCK": OutputType.LIST_ITEM,
        "LISTMENU_MESSAGE": OutputType.LISTMENU,
        "LISTMENU_BLOCK": OutputType.LISTMENU,
        "QUICKREPLY_BUTTON_MESSAGE": OutputType.QUICKREPLY,
        "QUICKREPLY_BUTTON_BLOCK": OutputType.QUICKREPLY,
    }

    __ACTION_TYPE_MAP: dict[str, ActionType] = {
        "TEXT_INPUT": ActionType.TEXT,
        "CARD_BUTTON_MESSAGE": ActionType.MESSAGE,
        "CARD_BUTTON_BLOCK": ActionType.BLOCK,
        "LIST_ITEM_MESSAGE": ActionType.MESSAGE,
        "LIST_ITEM_BLOCK": ActionType.BLOCK,
        "LISTMENU_MESSAGE": ActionType.MESSAGE,
        "LISTMENU_BLOCK": ActionType.BLOCK,
        "QUICKREPLY_BUTTON_MESSAGE": ActionType.MESSAGE,
        "QUICKREPLY_BUTTON_BLOCK": ActionType.BLOCK,
    }

    @property
    def output_type(self) -> OutputType:
        """TriggerType의 출력 유형을 반환합니다.

        이 메서드는 TriggerType Enum 멤버에 해당하는 출력 유형을 반환합니다.
        예를 들어, TriggerType.CARD_BUTTON_BLOCK은 OutputType.CARD_BUTTON을 반환합니다.
        만약 Enum 값이 매핑에 존재하지 않는 경우, ValueError를 발생시킵니다.

        Returns:
            OutputType: TriggerType에 해당하는 출력 유형입니다.

        Raises:
            ValueError: Enum 값이 매핑에 존재하지 않는 경우.
        """
        return self.__OUTPUT_TYPE_MAP[self.value]

    @property
    def action_type(self) -> ActionType:
        """TriggerType의 동작 유형을 반환합니다.

        이 메서드는 TriggerType Enum 멤버에 해당하는 동작 유형을 반환합니다.
        예를 들어, TriggerType.CARD_BUTTON_BLOCK은 ActionType.BLOCK을 반환합니다.
        만약 Enum 값이 매핑에 존재하지 않는 경우, ValueError를 발생시킵니다.

        Returns:
            ActionType: TriggerType에 해당하는 동작 유형입니다.

        Raises:
            ValueError: Enum 값이 매핑에 존재하지 않는 경우.
        """
        return self.__ACTION_TYPE_MAP[self.value]


class Block(ParentPayload):
    """flow의 lastBlock 및 trigger.referrerBlock 정보를 저장하는 클래스입니다.

    이 클래스는 카카오 챗봇의 flow 필드 내 block 정보를 표현합니다. block 정보는
    직전에 실행된 블록(lastBlock)이나 사용자가 상호작용한 블록(trigger.referrerBlock)의
    id 및 name을 포함합니다.

    Attributes:
        id (str): 블록의 고유 식별자
        name (str): 블록의 이름

    Examples:
        >>> block = Block.from_dict({"id": "block_id", "name": "block_name"})
        >>> block.id
        'block_id'
        >>> block.name
        'block_name'
    """

    def __init__(self, ID: str, name: str):
        """Block 객체를 생성합니다.

        Args:
            ID (str): 블록의 고유 식별자
            name (str): 블록 이름
        """
        self.id = ID
        self.name = name

    @classmethod
    def from_dict(cls, data: Dict) -> "Block":
        """딕셔너리에서 Block 객체를 생성합니다.

        Args:
            data (dict): 'id'와 'name' 키를 포함하는 블록 정보

        Returns:
            Block: 생성된 Block 객체
        """
        ID = data.get("id", "")
        name = data.get("name", "")
        return cls(ID, name)


class Trigger(ParentPayload):
    """flow의 trigger 정보를 저장하는 클래스입니다.

    Trigger는 사용자의 발화를 발생시킨 트리거 정보를 담습니다.
    여기에는 트리거의 유형(type: TriggerType)과 상호작용한 블록(referrerBlock)이 포함됩니다.

    Attributes:
        type (TriggerType): 발화를 발생시킨 트리거 유형 (OutputType + ActionType 조합)
        referrer_block (Block): 사용자가 상호작용한 블록 정보

    Examples:
        >>> trigger = Trigger.from_dict({
        ...     "type": "CARD_BUTTON_BLOCK",
        ...     "referrerBlock": {
        ...         "id": "ref_block_id",
        ...         "name": "ref_block_name"
        ...     }
        ... })
        >>> trigger.type.value
        'CARD_BUTTON_BLOCK'
        >>> trigger.type.output_type
        <OutputType.CARD_BUTTON: 'CARD_BUTTON'>
        >>> trigger.type.action_type
        <ActionType.BLOCK: 'BLOCK'>
        >>> trigger.referrer_block.id
        'ref_block_id'
    """

    def __init__(self, TYPE: TriggerType, referrer_block: Block):
        """Trigger 객체를 생성합니다.

        Args:
            TYPE (TriggerType): 트리거 유형
            referrer_block (Block): 사용자가 상호작용한 참조 블록 정보
        """
        self.type = TYPE
        self.referrer_block = referrer_block

    @classmethod
    def from_dict(cls, data: Dict) -> "Trigger":
        """딕셔너리에서 Trigger 객체를 생성합니다.

        Args:
            data (dict): 'type'과 'referrerBlock'을 포함한 트리거 정보

        Returns:
            Trigger: 생성된 Trigger 객체

        Raises:
            InvalidPayloadError: 유효하지 않은 TriggerType인 경우
        """
        raw_type = data.get("type", "")
        trigger_type = TriggerType(raw_type)
        referrer_block = Block.from_dict(data.get("referrerBlock", {}))
        return cls(trigger_type, referrer_block)


class Flow(ParentPayload):
    """Payload 객체의 'flow' 필드를 객체화한 클래스입니다.

    flow는 사용자와 챗봇의 대화 흐름 정보를 담고 있으며,
    트리거(trigger)와 마지막 블록(lastBlock)의 정보를 포함합니다.

    Attributes:
        trigger (Trigger): 발화를 발생시킨 트리거 정보
        last_block (Block): 직전에 실행된 블록 정보

    Example JSON:
        {
            "flow": {
                "trigger": {
                    "type": "CARD_BUTTON_BLOCK",
                    "referrerBlock": {
                        "id": "block_id",
                        "name": "block_name"
                    }
                },
                "lastBlock": {
                    "id": "prev_block_id",
                    "name": "prev_block_name"
                }
            }
        }

    Examples:
        >>> flow = Flow.from_dict({
        ...     "trigger": {
        ...         "type": "CARD_BUTTON_BLOCK",
        ...         "referrerBlock": {"id": "ref_block_id", "name": "ref_block_name"}
        ...     },
        ...     "lastBlock": {"id": "last_block_id", "name": "last_block_name"}
        ... })
        >>> flow.trigger.type
        <TriggerType.CARD_BUTTON_BLOCK: 'CARD_BUTTON_BLOCK'>
        >>> flow.trigger.type.output_type
        <OutputType.CARD_BUTTON: 'CARD_BUTTON'>
        >>> flow.trigger.type.action_type
        <ActionType.BLOCK: 'BLOCK'>
        >>> flow.last_block.name
        'last_block_name'
    """

    def __init__(self, trigger: Trigger, last_block: Block):
        """Flow 객체를 생성합니다.

        Args:
            trigger (Trigger): 발화를 유발한 트리거 정보
            last_block (Block): 직전에 실행된 블록 정보
        """
        self.trigger = trigger
        self.last_block = last_block

    @classmethod
    def from_dict(cls, data: Dict) -> "Flow":
        """딕셔너리에서 Flow 객체를 생성합니다.

        Args:
            data (dict): 'trigger'와 'lastBlock' 정보를 포함하는 flow 데이터

        Returns:
            Flow: 생성된 Flow 객체
        """
        trigger = Trigger.from_dict(data.get("trigger", {}))
        last_block = Block.from_dict(data.get("lastBlock", {}))
        return cls(trigger, last_block)


class ValidationPayload(ParentPayload):
    """ValidationPayload 객체를 저장하는 클래스입니다.

    파라미터를 검증하는 Kakao 스킬 서버의 요청을 처리하기 위한 Payload 객체입니다.
    이 클래스는 Bot, User 클래스를 속성으로 가지고 있습니다.
    이 클래스는 스킬 서버에서 전달받은 JSON 데이터를 객체로 변환합니다.

    Attributes:
        bot (Bot): 봇 정보를 담고 있는 객체
        is_in_slot_filling (bool): 슬롯 필링 중인지 여부
        lang (str): 사용자의 언어
        params (dict[str, str]): 파라미터 정보
        timezone (str): 사용자의 시간대
        user (User): 사용자 정보를 담고 있는 객체
        utterance (str): 사용자 발화
        value (dict[str, str]): 발화에서 추출한 값 정보
    """

    def __init__(
        self,
        bot: Bot,
        is_in_slot_filling: bool,
        lang: str,
        params: dict[str, str],
        timezone: str,
        user: User,
        utterance: str,
        value: dict[str, str],
    ):
        """ValidationPayload 인스턴스를 초기화합니다.

        스킬 서버의 요청을 처리하기 위한 사용자 정보와 관련된 속성을 초기화합니다.

        Args:
            bot (Bot): 봇 정보를 담고 있는 객체.
            is_in_slot_filling (bool): 슬롯 필링 중인지 여부.
            lang (str): 사용자의 언어.
            params (Dict[str, str]): 파라미터 정보.
            timezone (str): 사용자의 시간대.
            user (User): 사용자 정보를 담고 있는 객체.
            utterance (str): 사용자 발화.
            value (Dict[str, str]): 발화에서 추출한 값 정보.
        """
        self.bot = bot
        self.is_in_slot_filling = is_in_slot_filling
        self.lang = lang
        self.params = params
        self.timezone = timezone
        self.user = user
        self.utterance = utterance
        self.value = value

    @classmethod
    def from_dict(cls, data: Dict) -> "ValidationPayload":
        """딕셔너리에서 ValidationPayload 객체를 생성하는 클래스 메서드입니다.

        딕셔너리에서 각 키에 해당하는 값을 추출하여 ValidationPayload 객체를 생성합니다.
        self.bot, self.user에 대한 정보는
        Bot, User 클래스의 from_dict 메서드를 호출하여 객체로 변환합니다.

        Args:
            data (dict): ValidationPayload 객체를 생성하기 위한 딕셔너리

        Returns:
            ValidationPayload: 생성된 ValidationPayload 객체
        """
        bot = Bot.from_dict(data.get("bot", {}))
        is_in_slot_filling = data.get("isInSlotFilling", False)
        lang = data.get("lang", "")
        params = data.get("params", {})
        timezone = data.get("timezone", "")
        user = User.from_dict(data.get("user", {}))
        utterance = data.get("utterance", "")
        value = data.get("value", {})
        return cls(
            bot=bot,
            is_in_slot_filling=is_in_slot_filling,
            lang=lang,
            params=params,
            timezone=timezone,
            user=user,
            utterance=utterance,
            value=value,
        )

    @classmethod
    def from_json(cls, data: str) -> "ValidationPayload":
        """JSON 문자열을 Payload 객체로 변환하는 클래스 메서드입니다.

        전달받는 json 문자열은 skill server에서 전달받은 전체 data입니다.
        from_dict 메서드를 호출하여 JSON 문자열을 객체로 변환합니다.

        Args:
            data (str): JSON 문자열
        """
        return cls.from_dict(json.loads(data))


class Payload(ParentPayload):
    """Payload 객체를 저장하는 클래스입니다.

    Payload 객체는 스킬 실행시 봇 시스템이 스킬 서버에게 전달하는 정보를 객체화한 것입니다.
    이 클래스는 Intent, UserRequest, Bot, Action, Flow 객체를 속성으로 가지고 있습니다.
    스킬 서버에서 전달받은 json 데이터 또는 dict 데이터를 곧바로 Payload 객체로 변환할 수 있습니다.

    Examples:
        >>> response_json = {
        ...     "intent": {
        ...         "id": "intent_id",
        ...         "name": "intent_name",
        ...         "extra": {
        ...             "reason": {},
        ...             "matched_knowledges": []
        ...         }
        ...     },
        ...     "userRequest": {
        ...         "timezone": "Asia/Seoul",
        ...         "block": {
        ...             "id": "block_id",
        ...             "name": "block_name"
        ...         },
        ...         "utterance": "사용자 발화",
        ...         "lang": "ko",
        ...         "user": {
        ...             "id": "user_id",
        ...             "type": "botUserKey",
        ...             "properties": {}
        ...         },
        ...         "params": {},
        ...         "callbackUrl": "https://www.example.com"
        ...     },
        ...     "bot": {
        ...         "id": "bot_id",
        ...         "name": "bot_name"
        ...     },
        ...     "action": {
        ...         "id": "action_id",
        ...         "name": "action_name",
        ...         "params": {},
        ...         "detailParams": {},
        ...         "clientExtra": {}
        ...     },
        ...     "contexts": [],
        ...     "flow": {
        ...         "trigger": {
        ...             "type": "CARD_BUTTON_BLOCK",
        ...             "referrerBlock": {
        ...                 "id": "ref_id",
        ...                 "name": "ref_name"
        ...             }
        ...         },
        ...         "lastBlock": {
        ...             "id": "last_id",
        ...             "name": "last_name"
        ...         }
        ...     }
        ... }
        >>> payload = Payload.from_dict(response_json)
        >>> payload.flow.trigger.type
        <TriggerType.CARD_BUTTON_BLOCK: 'CARD_BUTTON_BLOCK'>

    Attributes:
        intent (Intent): 발화와 일치하는 블록이나 지식의 정보를 담고 있는 객체
        user_request (UserRequest): 사용자의 요청 정보를 담고 있는 객체
        bot (Bot): 봇 정보를 담고 있는 객체
        action (Action): 사용자의 요청에 대한 액션 정보를 담고 있는 객체
        contexts (list[Context]): 컨텍스트 정보를 담고 있는 객체 배열
        flow (Flow): 사용자와 챗봇의 대화 흐름 정보를 담고 있는 객체

    Properties:
        user_id (str): 사용자의 ID == user_request.user.id
        params (Params): 액션에 포함된 파라미터 정보 == action.params
        utterance (str): 사용자 발화
    """

    def __init__(
        self,
        intent: Intent,
        user_request: UserRequest,
        bot: Bot,
        action: Action,
        contexts: Optional[List[Context]] = None,
        flow: Optional[Flow] = None,
    ):
        """Payload 객체를 생성하는 메서드입니다.

        Args:
            intent (Intent): 발화와 일치하는 블록이나 지식의 정보를 담고 있는 객체
            user_request (UserRequest): 사용자의 요청 정보를 담고 있는 객체
            bot (Bot): 봇 정보를 담고 있는 객체
            action (Action): 사용자의 요청에 대한 액션 정보를 담고 있는 객체
            contexts (list[Context]): 컨텍스트 정보를 담고 있는 객체 배열
            flow (Flow): 사용자와 챗봇의 대화 흐름 정보 객체
        """
        self.intent = intent
        self.user_request = user_request
        self.bot = bot
        self.action = action
        self.flow = flow
        if contexts is None:
            contexts = []
        self.contexts = contexts

    @classmethod
    def from_dict(cls, data: Dict) -> "Payload":
        """딕셔너리에서 Payload 객체를 생성하는 클래스 메서드입니다.

        전달받는 딕셔너리는 skill server에서 전달받은 전체 data입니다.

        Args:
            data (dict): payload 정보가 담긴 딕셔너리

        Returns:
            Payload: 생성된 Payload 객체
        """
        intent = Intent.from_dict(data.get("intent", {}))
        user_request = UserRequest.from_dict(data.get("userRequest", {}))
        bot = Bot.from_dict(data.get("bot", {}))
        action = Action.from_dict(data.get("action", {}))
        flow = Flow.from_dict(data.get("flow", {}))
        # flow = data.get("flow", {})
        contexts = [Context.from_dict(context) for context in data.get("contexts", [])]
        return cls(intent, user_request, bot, action, contexts, flow)

    @classmethod
    def from_json(cls, data: str) -> "Payload":
        """JSON 문자열을 Payload 객체로 변환하는 클래스 메서드입니다.

        전달받는 json 문자열은 skill server에서 전달받은 전체 data입니다.
        from_dict 메서드를 호출하여 JSON 문자열을 객체로 변환합니다.

        Args:
            data (str): JSON 문자열
        """
        return cls.from_dict(json.loads(data))

    @property
    def user_id(self) -> Optional[str]:
        """사용자의 ID를 반환합니다.

        사용자의 ID는 user_request.user.id로부터 가져옵니다.

        Returns:
            str: 사용자의 ID
        """
        if (
            hasattr(self, "user_request")
            and hasattr(self.user_request, "user")
            and hasattr(self.user_request.user, "id")
        ):
            return self.user_request.user.id
        return None

    @property
    def params(self) -> Optional[Dict[str, str]]:
        """액션에 포함된 파라미터 정보를 반환합니다.

        액션에 포함된 파라미터 정보는 action.params로부터 가져옵니다.

        Returns:
            dict: 액션에 포함된 파라미터 정보
            None: 액션이 없거나 파라미터 정보가 없는 경우
        """
        if hasattr(self, "action") and hasattr(self.action, "params"):
            return self.action.params
        return None

    @property
    def detail_params(self) -> Optional[Dict[str, Param]]:
        """액션에 포함된 파라미터 세부 정보를 반환합니다.

        액션에 포함된 파라미터 세부 정보는 action.detail_params로부터 가져옵니다.

        Returns:
            dict: 액션에 포함된 파라미터 세부 정보
            None: 액션이 없거나 파라미터 세부 정보가 없는 경우
        """
        if hasattr(self, "action") and hasattr(self.action, "detail_params"):
            return self.action.detail_params
        return None

    @property
    def utterance(self) -> Optional[str]:
        """사용자 발화를 반환합니다.

        사용자 발화는 user_request.utterance로부터 가져옵니다.

        Returns:
            str: 사용자 발화
            None: 사용자 발화 정보가 없는 경우
        """
        if hasattr(self, "user_request") and hasattr(self.user_request, "utterance"):
            return self.user_request.utterance
        return None



"""카카오톡 챗봇을 개발하기 위한 라이브러리입니다.

Packages:
    response: 카카오톡 응답 규칙에 맞는 응답 객체들을 정의합니다.

Modules:
    base: 라이브러리의 기본 부모 클래스(BaseModel)와 스킬 템플릿(SkillTemplate)을 정의합니다.
    context: 컨텍스트 정보를 담는 클래스를 정의합니다.
    customerror: 사용자 정의 예외를 정의합니다.
    input: 스킬 실행시 봇 시스템이 스킬 서버에게 전달하는 정보를 객체화한 Payload 객체들을 정의합니다.
    utils: 라이브러리에서 사용하는 유틸리티 함수들을 정의합니다.
    validation: 라이브러리에서 사용하는 유효성 검사 함수들을 정의합니다.
"""

from . import response
from .customerror import InvalidLinkError, InvalidTypeError, InvalidActionError
from .input import Payload, ValidationPayload

__all__ = [
    "response",
    "InvalidActionError",
    "InvalidLinkError",
    "InvalidTypeError",
    "Payload",
    "ValidationPayload",
]



"""카카오톡 출력 요소 Card 관련 클래스를 정의한 모듈입니다.

카카오톡 출력 요소 Card는 텍스트, 이미지, 버튼 등을 포함하는 카드 컴포넌트입니다.

Classes:
    - TextCardComponent: 카카오톡 출력 요소 TextCard의 객체를 생성하는 클래스
    - BasicCardComponent: 카카오톡 출력 요소 BasicCard의 객체를 생성하는 클래스
    - CommerceCardComponent: 카카오톡 출력 요소 CommerceCard의 객체를 생성하는 클래스
    - ListCardComponent: 카카오톡 출력 요소 ListCard의 객체를 생성하는 클래스
    - ItemCardComponent: 카카오톡 출력 요소 ItemCard의 객체를 생성하는 클래스
"""

from abc import ABCMeta, abstractmethod
from typing import Optional, overload, Union, List, Dict


from ..base import ParentComponent
from .common import Button, Link, ListItem, Profile, Thumbnail
from ..interaction import ActionEnum
from .itemcard import ImageTitle, Item, ItemListSummary, ItemProfile, ItemThumbnail
from ...validation import validate_int, validate_str, validate_type

__all__ = [
    "TextCardComponent",
    "BasicCardComponent",
    "CommerceCardComponent",
    "ListCardComponent",
    "ItemCardComponent",
]


class ParentCardComponent(ParentComponent, metaclass=ABCMeta):
    """Component 출력 요소중 Card 종류의 부모 클래스입니다.

    Card 출력 요소는 TextCardComponent, BasicCardComponent,
    CommerceCardComponent, ListCardComponent, ItemCardComponent가 있습니다.
    이 클래스는 Card 출력 요소의 공통 속성과 메서드를 정의합니다.
    주로 Button 객체를 조작하는 메서드를 제공합니다.

    Attributes:
        buttons (list[Button], optional): 버튼 객체입니다.
    """

    def __init__(self, buttons: Optional[List[Button]] = None):
        """ParentCard 객체를 생성합니다.

        buttons가 None인 경우 빈 리스트로 초기화합니다.

        Args:
            buttons (Optional[list[Button]], optional): 버튼 객체.
        """
        if buttons is None:
            buttons = []
        self.buttons = buttons

    def validate(self):
        """객체가 카카오톡 출력 요소에 맞는지 확인합니다.

        Raises:
            InvalidTypeError: 받거나 생성한 Button 객체가 Button이 아닌 경우
        """
        super().validate()
        if self.buttons:
            for button in self.buttons:
                validate_type(Button, button)

    @overload
    def add_button(self, button: Button) -> "ParentCardComponent": ...

    @overload
    def add_button(
        self,
        label: str,
        action: Union[str, ActionEnum],
        web_link_url: Optional[str] = None,
        message_text: Optional[str] = None,
        phone_number: Optional[str] = None,
        block_id: Optional[str] = None,
        extra: Optional[Dict] = None,
    ) -> "ParentCardComponent": ...

    def add_button(self, *args, **kwargs) -> "ParentCardComponent":
        """버튼을 추가합니다.

        Button 객체 또는 Button 생성 인자를 받아 버튼 리스트에 추가합니다.

        Args:
            *args (Button | str | ActionEnum | Dict[str, Any]): 추가할 Button 객체 또는 Button 생성 인자.
                - Button 객체를 직접 전달할 수 있습니다.
                - Button 생성에 필요한 인자들을 전달할 수 있습니다.
            **kwargs (Any): Button 생성 인자.
                - label (str): 버튼에 적히는 문구입니다.
                - action (str | ActionEnum): 버튼 클릭 시 수행될 작업입니다. ('message', 'block', 등)
                - web_link_url (Optional[str]): 웹 브라우저를 열고 이동할 주소입니다. (action이 'webLink'일 경우 필수)
                - message_text (Optional[str]): action이 'message'인 경우 사용자 측으로 발화될 텍스트입니다.
                - phone_number (Optional[str]): action이 'phone'일 경우 전화번호입니다.
                - block_id (Optional[str]): action이 'block'일 경우 호출할 block의 ID입니다.
                - extra (Optional[dict]): 스킬 서버에 추가로 전달할 데이터입니다.

        Returns:
            ParentCardComponent: Button 객체가 추가된 ParentCardComponent 객체.

        Raises:
            InvalidTypeError: 받거나 생성한 Button 객체가 Button이 아닌 경우.
        """
        if len(args) == 1 and isinstance(args[0], Button):
            self.buttons.append(args[0])
        elif len(args) == 0 and "button" in kwargs:
            self.buttons.append(kwargs["button"])
        else:
            button = Button(*args, **kwargs)
            self.buttons.append(button)
        return self

    def remove_button(self, button: Button):
        """버튼을 제거합니다.

        Button 객체를 받아 버튼 리스트에서 제거합니다.

        Parameters:
            button: 제거할 버튼 객체
        """
        self.buttons.remove(button)

    @abstractmethod
    def render(self): ...


class TextCardComponent(ParentCardComponent):
    """카카오톡 출력 요소 TextCard의 객체를 생성하는 클래스입니다.

    TextCardComponent는 제목과 설명을 출력하는 요소입니다.
    텍스트 카드는 간단한 텍스트에 버튼을 추가하거나, 텍스트를 케로셀형으로 전달하고자 할 때 사용됩니다.
    title과 description 중 최소 하나는 None이 아니어야 합니다.

    Args:
        title (Optional[str], optional): 카드 제목. Defaults to None.
        description (Optional[str], optional): 카드 설명. Defaults to None.
        buttons (Optional[list[Button]], optional): 버튼 리스트. Defaults to None.

    Examples:
        >>> text_card = TextCardComponent(title="제목", description="설명")
        >>> text_card.add_button(
                label="버튼 1", action="message", message_text="버튼 1 클릭"
            )
        >>> text_card.render()
        {
            "title": "제목",
            "description": "설명",
            "buttons": [
                {
                    "label": "버튼 1",
                    "action": "message",
                    "messageText": "버튼 1 클릭"
                }
            ]
        }
    """

    name = "textCard"

    def __init__(
        self,
        title: Optional[str] = None,
        description: Optional[str] = None,
        buttons: Optional[List[Button]] = None,
    ):
        """TextCardComponent 객체를 생성합니다.

        title과 description 중 최소 하나는 None이 아니어야 합니다.

        Args:
            title (Optional[str], optional): 카드의 제목.
            description (Optional[str], optional): 카드 상세 설명.
            buttons (Optional[list[Button]], optional): 카드의 버튼들.
        """
        super().__init__(buttons=buttons)
        self.title = title
        self.description = description

    def validate(self):
        """객체가 카카오톡 출력 요소에 맞는지 확인합니다.

        title과 description 중 최소 하나는 None이 아니어야 합니다.

        raise:
            ValueError: title과 description 중 최소 하나는 None이 아니어야 합니다.
        """
        super().validate()
        if self.title is None and self.description is None:
            raise ValueError(
                "title과 description 중 최소 하나는 None이 아니어야 합니다."
            )
        if self.title is None:
            validate_str(self.description)
        else:
            validate_str(self.title)

    def render(self):
        """객체의 응답 내용을 반환합니다.

        TextCardComponent 객체의 응답 내용을 반환합니다.
        이 응답 내용을 이용하여 render() 메서드에서 최종 응답을 생성합니다.

        Returns:
            dict: 응답 내용

        Examples:
            >>> self.render()
            {
                "title": "제목", # 제목이 없을 경우 미표시
                "description": "설명", # 설명이 없을 경우 미표시
                "buttons": buttons.get_response_content()
            }
        """
        self.validate()
        response = {
            "title": self.title,
            "description": self.description,
            "buttons": (
                [button.render() for button in self.buttons] if self.buttons else None
            ),
        }
        return self.remove_none_item(response)


class BasicCardComponent(ParentCardComponent):
    """카카오톡 출력 요소 BasicCard의 객체를 생성하는 클래스입니다.

    BasicCardComponent는 소셜, 썸네일, 프로필 등을 통해서 사진이나 글, 인물 정보 등을 공유할 수 있습니다.
    제목과 설명 외에 썸네일 그룹, 프로필, 버튼 그룹, 소셜 정보를 추가로 포함합니다.
    Thumbnail 객체는 반드시 포함되어야 합니다. 제목과 설명만 있는 TextCardComponent와의 차이점입니다.

    Args:
        thumbnail (Thumbnail): 카드의 상단 이미지
        title (Optional[str] optional): 카드 제목. Defaults to None.
        description (Optional[str] optional): 카드 상세 설명. Defaults to None.
        buttons (Optional[list[Button]], optional): 버튼 리스트. Defaults to None.

    Examples:
        >>> basic_card = BasicCardComponent(
        ...     thumbnail=Thumbnail(
        ...         image_url="https://example.com/image.jpg",
        ...         link=Link(web="https://example.com")
        ...     ),
        ...     title="제목",
        ...     description="설명"
        ... )
        >>> basic_card.add_button(
        ...     label="버튼 1", action="message", message_text="버튼 1 클릭"
        ... )
        >>> basic_card.render()
        {
            "thumbnail": {
                "imageUrl": "https://example.com/image.jpg",
                "link": {
                    "web": "https://example.com"
                }
            },
            "title": "제목",
            "description": "설명",
            "buttons": [
                {
                    "label": "버튼 1",
                    "action": "message",
                    "messageText": "버튼 1 클릭"
                }
            ]
        }
    """

    name = "basicCard"

    def __init__(
        self,
        thumbnail: Thumbnail,
        title: Optional[str] = None,
        description: Optional[str] = None,
        buttons: Optional[List[Button]] = None,
        forwardable: bool = False,
    ):
        """BasicCardComponent 객체를 생성합니다.

        Args:
            thumbnail (Thumbnail): 카드의 상단 이미지
            title (str, optional): 카드 제목. Defaults to None.
            description (str optional): 카드 상세 설명. Defaults to None.
            buttons (list[Button], optional): 버튼 리스트. Defaults to None.
            forwardable (bool, optional): 공유하기 버튼 표시 여부. Defaults to False.
        """
        super().__init__(buttons=buttons)
        self.thumbnail = thumbnail
        self.title = title
        self.description = description
        self.forwardable = forwardable

    def validate(self):
        """객체가 카카오톡 출력 요소에 맞는지 확인합니다.

        Thumbnail 객체는 반드시 포함되어야 합니다.
        제목과 설명은 문자열이어야 합니다.

        raise:
            InvalidTypeError: Thumbnail 객체가 Thumbnail이 아닌 경우
            ValueError: title과 description이 문자열이 아니어야 합니다.
        """
        super().validate()
        validate_type(Thumbnail, self.thumbnail)
        validate_str(self.title, self.description)

    def render(self):
        """객체의 응답 내용을 반환합니다.

        BasicCard 객체의 응답 내용을 반환합니다.

        Examples:
            >>> self.render()
            {
                "thumbnail": thumbnail.get_response_content(),
                "title": "제목",
                "description": "설명",
                "buttons": buttons.get_response_content(),
                "forwardable": True
            }

        Returns:
            dict: 응답 내용
        """
        self.validate()
        response = {
            "thumbnail": self.thumbnail.render(),
            "title": self.title,
            "description": self.description,
            "buttons": (
                [button.render() for button in self.buttons] if self.buttons else None
            ),
            "forwardable": self.forwardable if self.forwardable else None,
        }
        return self.remove_none_item(response)


class CommerceCardComponent(ParentCardComponent):
    """카카오톡 출력 요소 CommerceCard의 객체를 생성하는 클래스입니다.

    CommerceCardComponent는 제품에 대한 소개, 구매 안내 등을 사용자에게 전달할 수 있습니다.
    커머스 카드는 제목과 설명 외에 썸네일 그룹, 프로필, 버튼 그룹, 가격 정보를 추가로 포함합니다.

    discountedPrice 가 존재하면 price, discount, discountRate 과 관계 없이
    무조건 해당 값이 사용자에게 노출됩니다.
    discountRate은 discountedPrice를 필요로 합니다.
    discountedPrice가 주어지지 않으면 사용자에게 >discountRate을 노출하지 않습니다.
    discountRate과 discount가 동시에 있는 경우, discountRate을 우선적으로 노출합니다.

    Examples:
        >>> commerce_card = CommerceCardComponent(
        ...     price=10999,
        ...     thumbnails=[
        ...         Thumbnail(image_url="https://example.com/image1.jpg"),
        ...     ],
        ...     title="커머스 카드",
        ...     description="커머스 카드 설명",
        ...     buttons=[
        ...             Button(
        ...                 label="구매하기", action="webLink",
        ...                 web_link_url="https://example.com")
        ...         ]
        ...     ),
        ...     profile=Profile(
        ...         nickname="제품 이름",
        ...         image_url="https://example.com/image.jpg"
        ...     ),
        ...     currency="won",
        ...     discount=1000,
        ...     discount_rate=10,
        ...     discount_price=9000
        ... )
        >>> commerce_card.render()
        {
            "price": 10999,
            "thumbnails": {
                "thumbnails": [
                    {
                        "imageUrl": "https://example.com/image1.jpg"
                    }
                ]
            },
            "title": "커머스 카드",
            "description": "커머스 카드 설명",
            "currency": "won",
            "discount": "1000",
            "discountRate": "10",
            "discountPrice": "9000",
            "profile": {
                "nickname": "제품 이름",
                "imageUrl": "https://example.com/image.jpg"
            },
            "buttons": {
                "buttons": [
                    {
                        "label": "구매하기",
                        "action": "webLink",
                        "webLinkUrl": "https://example.com"
                    }
                ]
            }
        }

    Attributes:
        price (int): 제품 가격
        thumbnails (list[Thumbnail]): 제품에 대한 사진(현재는 1개만 지원)
        title (Optional[str], optional): 제품의 이름. Defaults to None.
        description (Optional[str], optional): 제품에 대한 상세 설명. Defaults to None.
        buttons (Optional[list[Button]], optional): 버튼 리스트. Defaults to None.
        profile (Optional[Profile], optional): 제품을 판매하는 프로필. Defaults to None.
        currency (Optional[str], optional): 제품을 가격의 통화(현재는 won만 가능).
            Defaults to None.
        discount (Optional[str], optional): 제품의 가격에 대한 할인할 금액.
            Defaults to None.
        discount_rate (Optional[str], optional): 제품의 가격에 대한 할인율.
            Defaults to None.
        discount_price (Optional[str], optional): 제품의 가격에 대한 할인가(할인된 가격).
            discountRate을 쓰는 경우 필수.
            Defaults to None.


    """

    name = "commerceCard"

    def __init__(
        self,
        price: int,
        thumbnails: Union[List[Thumbnail], Thumbnail],
        title: Optional[str] = None,
        description: Optional[str] = None,
        buttons: Optional[List[Button]] = None,
        profile: Optional[Profile] = None,
        currency: Optional[str] = None,
        discount: Optional[int] = None,
        discount_rate: Optional[int] = None,
        discount_price: Optional[int] = None,
    ):
        """CommerceCardComponent 객체를 생성합니다."""
        super().__init__(buttons=buttons)
        self.price = price
        if isinstance(thumbnails, Thumbnail):
            thumbnails = [thumbnails]
        self.thumbnails = thumbnails
        self.title = title
        self.description = description
        self.currency = currency
        self.discount = discount
        self.discount_rate = discount_rate
        self.discount_price = discount_price
        self.profile = profile

    def validate(self):
        """객체가 카카오톡 출력 요소에 맞는지 확인합니다.

        raise:
            ValueError: price가 int가 아닌 경우
            ValueError: title, description이 문자열이 아닌 경우
            AssertionError: currency가 "won"이 아닌 경우
            ValueError: discount, discount_price, discount_rate가 int가 아닌 경우
            InvalidTypeError: thumbnails의 각 요소가 Thumbnail이 아닌 경우
            InvalidTypeError: profile이 Profile이 아닌 경우
        """
        super().validate()
        validate_int(self.price, disallow_none=True)
        for thumbnail in self.thumbnails:
            validate_type(Thumbnail, thumbnail)
        validate_str(self.title, self.description)
        if self.currency:
            assert self.currency == "won"
        validate_int(
            self.discount, self.discount_price, self.discount_rate, self.discount_price
        )
        validate_type(Profile, self.profile)

    def render(self) -> Dict:
        """객체의 응답 내용을 반환합니다.

        CommerceCardComponent 객체의 응답 내용을 반환합니다.
        None인 속성은 미표시됩니다.

        Returns:
            dict: 응답 내용

        Examples:
            >>> self.render()
            {
                "price": 10999,
                "thumbnails": [
                    thumbnail.render() for thumbnail in self.thumbnails
                ],
                "title": "커머스 카드",
                "description": "커머스 카드 설명",
                "currency": "won",
                "discount": 1000,
                "discountRate": 10,
                "discountPrice": 9000,
                "profile": profile.render(),
                "buttons": [button.render() for button in self.buttons]
            }
        """
        self.validate()
        response = {
            "price": self.price,
            "thumbnails": [thumbnail.render() for thumbnail in self.thumbnails],
            "title": self.title,
            "description": self.description,
            "currency": self.currency,
            "discount": self.discount,
            "discountRate": self.discount_rate,
            "discountPrice": self.discount_price,
            "profile": self.profile.render() if self.profile else None,
            "buttons": (
                [button.render() for button in self.buttons] if self.buttons else None
            ),
        }
        return self.remove_none_item(response)


class ListCardComponent(ParentCardComponent):
    """카카오톡 출력 요소 ListCard의 객체를 생성하는 클래스입니다.

    ListCardComponent는 헤더와 아이템을 포함하며, 헤더는 리스트 카드의 상단에 위치합니다.
    리스트 상의 아이템은 각각의 구체적인 형태를 보여주며, 제목과 썸네일, 상세 설명을 포함합니다.
    header는 ListItem 객체이어야 하지만, str으로 입력받을 경우 ListItem으로 변환됩니다.

    Examples:
        >>> list_card = ListCardComponent(
        ...     header="리스트 카드"
        ... )
        >>> list_card.add_item(
        ...     ListItem(
        ...         title="아이템 1",
        ...         description="아이템 1 설명",
        ...         image_url="https://example.com/image1.jpg"
        ...     )
        ... )
        >>> list_card.render()
        {
            "header": {
                "title": "리스트 카드"
            },
            "items": [
                {
                    "title": "아이템 1",
                    "description": "아이템 1 설명",
                    "imageUrl": "https://example.com/image1.jpg"
                }
            ]
        }

    Attributes:
        header (ListItem): 리스트 카드의 헤더
        items (ListItems): 리스트 카드의 아이템들
        max_buttons (int): 버튼 최대 개수. Defaults to 2.
        max_items (int): 아이템 최대 개수. Defaults to 5.
            ListCardComponent가 Carousel로 나타나는 경우 최대 4개로 제한되어 있기 때문에,
            값을 4로 수정해야 합니다.
    """

    name = "listCard"

    def __init__(
        self,
        header: Union[ListItem, str],
        items: Optional[List[ListItem]] = None,
        buttons: Optional[List[Button]] = None,
        max_buttons: int = 2,
        max_items: int = 5,
    ):
        """ListCard 객체를 생성합니다.

        Args:
            header (Union[ListItem, str]): 리스트 카드의 상단 항목 str인 경우 ListItem으로 변환됩니다.
            items (ListItems): 리스트 카드의 각각 아이템
            buttons (Optional[list[Button]], optional): 리스트 카드의 버튼들.
                                                    Defaults to None.
            max_buttons (int, optional): 버튼 최대 개수. Defaults to 2.
            max_items (int, optional): 아이템 최대 개수. Defaults to 5.
        """
        super().__init__(buttons=buttons)
        if isinstance(header, str):
            self.header = ListItem(title=header)
        else:
            self.header = header
        if items is None:
            items = []
        self.items = items
        self.max_buttons = max_buttons
        self.max_items = max_items

    def validate(self):
        """객체가 카카오톡 출력 요소에 맞는지 확인합니다.

        raise:
            InvalidTypeError: header가 ListItem이 아닌 경우
            AssertionError: items가 비어있는 경우
            InvalidTypeError: items가 ListItems가 아닌 경우
        """
        super().validate()
        if len(self.items) < 1:
            raise AssertionError(
                "ListCardComponent는 최소 1개의 아이템을 포함해야 합니다."
            )
        if len(self.items) > self.max_items:
            raise AssertionError(
                f"ListCard는 최대 {self.max_items}개의 아이템을 포함할 수 있습니다."
            )
        if len(self.buttons) > self.max_buttons:
            raise AssertionError(
                f"ListCard는 최대 {self.max_buttons}개의 버튼을 포함할 수 있습니다."
            )
        validate_type(ListItem, self.header, disallow_none=True)
        validate_type(ListItem, *self.items)

    def render(self):
        """객체의 응답 내용을 반환합니다.

        ListCardComponent 객체의 응답 내용을 반환합니다.
        이 응답 내용을 이용하여 render() 메서드에서 최종 응답을 생성합니다.

        Examples:
            >>> self.render()
            {
                "header": header.get_response_content(),
                "items": items.get_response_content(),
                "buttons": buttons.get_response_content()
            }

        Returns:
            dict: 응답 내용
        """
        self.validate()
        response = {
            "header": self.header.render(),
            "items": ([item.render() for item in self.items] if self.items else None),
            "buttons": (
                [button.render() for button in self.buttons] if self.buttons else None
            ),
        }
        return self.remove_none_item(response)

    @overload
    def add_item(
        self,
        title: str,
        description: Optional[str] = None,
        image_url: Optional[str] = None,
        link: Optional[Link] = None,
        action: Optional[Union[str, ActionEnum]] = None,
        block_id: Optional[str] = None,
        message_text: Optional[str] = None,
        extra: Optional[Dict] = None,
    ) -> "ListCardComponent": ...

    @overload
    def add_item(self, item: ListItem) -> "ListCardComponent": ...

    def add_item(self, *args, **kwargs) -> "ListCardComponent":
        """ListCardComponent에 아이템을 추가합니다.

        ListItem 객체 또는 ListItem 생성 인자를 받아 아이템 리스트에 추가합니다.

        Args:
            *args: ListItem 생성 인자 또는 ListItem 객체
            **kwargs: ListItem 생성 인자 또는 ListItem 객체

        Returns:
            ListCardComponent: ListItem이 추가된 객체
        """
        if len(args) == 1 and isinstance(args[0], ListItem):
            self.items.append(args[0])
        elif len(args) == 0 and "item" in kwargs:
            self.items.append(kwargs["item"])
        else:
            item = ListItem(*args, **kwargs)
            self.items.append(item)
        return self

    def remove_item(self, item: ListItem) -> "ListCardComponent":
        """ListCardComponent에서 아이템을 제거합니다.

        Args:
            item (ListItem): 제거할 ListItem 객체
        """
        assert isinstance(item, ListItem)
        if item not in self.items:
            raise ValueError("해당 아이템이 존재하지 않습니다.")
        self.items.remove(item)

        return self


class ItemCardComponent(ParentCardComponent):
    """카카오톡 출력 요소 ItemCard의 객체를 생성하는 클래스입니다.

    ItemCardComponent는 메시지 목적에 따른 유관 정보들을 (가격 정보 포함)
    사용자에게 일목요연한 리스트 형태로 전달할 수 있습니다.
    ItemCardComponent는 아이템리스트, 제목, 설명 외에
    썸네일, 프로필, 헤드, 이미지타이틀, 버튼 그룹을 추가로 포함합니다.

    Examples:
        >>> item_card = ItemCardComponent(
        ...     item_list=[
        ...         Item(title="아이템 1", description="아이템 1 설명"),
        ...         Item(title="아이템 2", description="아이템 2 설명")
        ...     ],
        ...     thumbnail=ItemThumbnail(
        ...         image_url="https://example.com/image.jpg"
        ...     ),
        ...     head="헤드",
        ...     profile=ItemProfile(
        ...         title="프로필",
        ...         image_url="https://example.com/image.jpg"
        ...     ),
        ...     image_title=ImageTitle(
        ...         title="이미지 타이틀",
        ...         description="이미지 타이틀 설명",
        ...         image_url="https://example.com/image.jpg"
        ...     ),
        ...     item_list_alignment="left",
        ...     item_list_summary=ItemListSummary(
        ...         price=10000
        ...     ),
        ...     title="타이틀",
        ...     description="설명",
        ...     buttons=[
        ...         Button(
        ...             label="버튼 1",
        ...             action="message",
        ...             message_text="버튼 1 클릭"
        ...         )
        ...     ],
        ...     button_layout="vertical"
        ... )
        >>> item_card.render()
        {
            "thumbnail": {
                "imageUrl": "https://example.com/image.jpg"
            },
            "head": {
                "title": "헤드"
            },
            "profile": {
                "title": "프로필",
                "imageUrl": "https://example.com/image.jpg"
            },
            "imageTitle": {
                "title": "이미지 타이틀",
                "description": "이미지 타이틀 설명",
                "imageUrl": "https://example.com/image.jpg"
            },
            "itemList": [
                {
                    "title": "아이템 1",
                    "description": "아이템 1 설명"
                },
                {
                    "title": "아이템 2",
                    "description": "아이템 2 설명"
                }
            ],
            "itemListAlignment": "left",
            "itemListSummary": {
                "price": 10000
            },
            "title": "타이틀",
            "description": "설명",
            "buttonLayout": "vertical",
            "buttons": [
                {
                    "label": "버튼 1",
                    "action": "message",
                    "messageText": "버튼 1 클릭"
                }
            ]
        }

    Attributes:
        item_list (list[Item]): Item 객체들이 포함된 리스트
        thumbnail (ItemThumbnail): 상단 이미지
            - 단일형: 이미지 비율 2:1 (800*400), 1:1 (800*800)사용 가능
            - 케로셀형: 이미지 비율 2:1 (800*400)만 사용 가능
        head (str): 헤드 정보입니다. 프로필과 함께 사용할 수 없습니다.
        profile (ItemProfile): 프로필 정보. 헤드와 함께 사용할 수 없습니다.
        image_title (ImageTitle): 이미지를 동반하는 제목 및 설명 정보입니다.
            이미지 우측 정렬 고정 (위치 변경 불가)
        item_list_alignment (str): itemList 및 itemListAlignment 정렬 정보입니다.
            (left, right 중 선택)
        item_list_summary (ItemListSummary): 아이템 가격 정보입니다.
            사용시 item_list_alignment는 right 권장.
        title (str): 타이틀 정보. description을 넣는 경우, title이 필수입니다.
        description (str): 설명. 사용시 title이 필수입니다.
        button_layout (str): 버튼 정렬 정보. 단일형에서만 사용 가능.
            (vertical, horizontal 중 선택)
            미입력시 자동 선택
        buttons (Optional[list[Button]], optional): 버튼 리스트. Defaults to None.
            - 단일형: 최대 3개까지 사용 가능
            - 케로셀형: 최대 2개까지 사용 가능
    """

    name = "itemCard"

    def __init__(
        self,
        item_list: List[Item],
        thumbnail: Optional[ItemThumbnail] = None,
        head: Optional[str] = None,
        profile: Optional[ItemProfile] = None,
        image_title: Optional[ImageTitle] = None,
        item_list_alignment: Optional[str] = None,
        item_list_summary: Optional[ItemListSummary] = None,
        title: Optional[str] = None,
        description: Optional[str] = None,
        buttons: Optional[List[Button]] = None,
        button_layout: Optional[str] = None,
    ):
        """ItemCardComponent 객체를 생성합니다.

        Args: class Docstring 참고
        """
        super().__init__(buttons=buttons)
        self.item_list = item_list
        self.thumbnail = thumbnail
        self.head = head
        self.profile = profile
        self.image_title = image_title
        self.item_list_alignment = item_list_alignment
        self.item_list_summary = item_list_summary
        self.title = title
        self.description = description
        self.button_layout = button_layout

    def validate(self):
        """객체가 카카오톡 출력 요소에 맞는지 확인합니다.

        raise:
            InvalidTypeError: item_list의 각 요소가 Item이 아닌 경우
            InvalidTypeError: thumbnail가 ItemThumbnail 아닌 경우
            InvalidTypeError: head가 Head가 아닌 경우
            InvalidTypeError: profile가 ItemProfile이 아닌 경우
            InvalidTypeError: image_title가 ImageTitle이 아닌 경우
            InvalidTypeError: item_list_summary가 ItemListSummary이 아닌 경우
            ValueError: title, description, button_layout이 문자열이 아닌 경우
        """
        super().validate()
        for item in self.item_list:
            validate_type(Item, item)
        validate_type(ItemThumbnail, self.thumbnail)
        validate_type(ItemProfile, self.profile)
        validate_type(ImageTitle, self.image_title)
        validate_type(ItemListSummary, self.item_list_summary)
        validate_str(self.head, self.title, self.description, self.button_layout)

    def render(self):
        """객체의 응답 내용을 반환합니다.

        ItemCardComponent 객체의 응답 내용을 반환합니다.
        이 응답 내용을 이용하여 render() 메서드에서 최종 응답을 생성합니다.
        None인 속성은 미반환 됩니다.

        Examples:
            >>> self.render()
            {
                "thumbnail": thumbnail.get_response_content(),
                "head": {"title": "헤드"},
                "profile": profile.get_response_content(),
                "imageTitle": image_title.get_response_content(),
                "itemList": [item.render() ...]
                "itemListAlignment": "left",
                "itemListSummary": item_list_summary.get_response_content(),
                "title": "타이틀",
                "description": "설명",
                "buttonLayout": "vertical",
                "buttons": buttons.get_response_content()
            }

        Returns:
            dict: 응답 내용
        """
        self.validate()
        assert self.item_list is not None

        response = {
            "thumbnail": self.thumbnail.render() if self.thumbnail else None,
            "head": {"title": self.head} if self.head else None,
            "profile": self.profile.render() if self.profile else None,
            "imageTitle": (self.image_title.render() if self.image_title else None),
            "itemList": [item.render() for item in self.item_list],
            "itemListAlignment": self.item_list_alignment,
            "itemListSummary": (
                self.item_list_summary.render() if self.item_list_summary else None
            ),
            "title": self.title,
            "description": self.description,
            "buttonLayout": self.button_layout,
            "buttons": (
                [button.render() for button in self.buttons] if self.buttons else None
            ),
        }
        return self.remove_none_item(response)

    @overload
    def add_item(self, item: Item): ...

    @overload
    def add_item(self, title: str, description: str): ...

    def add_item(self, *args, **kwargs):
        """아이템을 추가합니다.

        Item 객체 또는 Item 생성 인자를 받아 아이템 리스트에 추가합니다.

        Args:
            *args (Item | str): 추가할 Item 객체 또는 Item 생성 인자.
                - Item 객체를 직접 전달할 수 있습니다.
                - Item 생성에 필요한 인자들을 전달할 수 있습니다.
            **kwargs (Any): 추가할 Item 생성 인자.
                - item (Item): 추가할 Item 객체.
                - title (str): 아이템의 제목.
                - description (str): 아이템의 설명.

        Returns:
            ParentCardComponent: 아이템이 추가된 ParentCardComponent 객체.

        Raises:
            ValueError: Item 객체와 함께 다른 인자를 주는 경우.
            ValueError: `item` 키워드에 Item 객체가 아닌 것을 주거나,
                        `title` 또는 `description`이 누락된 경우.
        """
        if len(args) == 1:
            if isinstance(args[0], Item):
                self.item_list.append(args[0])
            elif "description" in kwargs:
                self.item_list.append(Item(*args, **kwargs))
        else:
            if "item" in kwargs:
                self.item_list.append(kwargs["item"])
            self.item_list.append(Item(*args, **kwargs))

    @overload
    def remove_item(self, item: Item): ...

    @overload
    def remove_item(self, index: int): ...

    def remove_item(self, *args, **kwargs):
        """아이템을 제거합니다.

        Item 객체 또는 인덱스를 받아 아이템 리스트에서 제거합니다.

        Args:
            *args (Item | int): 제거할 Item 객체 또는 아이템의 인덱스.
                - Item 객체를 직접 전달할 수 있습니다.
                - 아이템의 인덱스를 전달할 수 있습니다.
            **kwargs (Any): 제거할 Item 객체 또는 아이템의 인덱스를 키워드 인자로 전달할 수 있습니다.
                - `item` (Item): 제거할 Item 객체.
                - `index` (int): 제거할 아이템의 인덱스.

        Returns:
            SomeClass: 아이템이 제거된 `SomeClass` 객체.

        Raises:
            ValueError: Item 객체와 함께 다른 인자를 주는 경우.
            ValueError: `item` 키워드에 Item 객체가 아닌 것을 주거나,
                        `index` 키워드에 정수가 아닌 값을 주는 경우.
        """
        if isinstance(args[0], Item):
            self.item_list.remove(args[0])
        elif isinstance(args[0], int):
            self.item_list.pop(args[0])
        elif "item" in kwargs:
            self.item_list.remove(kwargs["item"])
        elif "index" in kwargs:
            self.item_list.pop(kwargs["index"])



"""라이브러리의 기본 공통 부모 클래스를 정의합니다.

classes:
    - BaseModel: 라이브러리의 기본 공통 부모 클래스
    - ParentPayload: Payload 객체의 부모 클래스
    - SkillTemplate: 카카오톡 스킬 응답중 SkilllTemplate을 객체로 생성하는 추상 클래스
"""

from abc import ABC, ABCMeta, abstractmethod
from typing import Union, List, Dict
import json


class BaseModel(ABC, metaclass=ABCMeta):
    """라이브러리의 기본 공통 부모 클래스로 라이브러리의 대부분의 클래스는 이 클래스를 상속받아 구현됩니다.

    - remove_none_item: None이 아닌 값만을 가진 dict를 생성합니다.
    - render: 객체를 카카오톡 응답 형식에 알맞게 dict로 변환합니다.
    - validate: 카카오톡 응답 규칙에 맞는지 검증합니다.
    """

    @staticmethod
    def remove_none_item(base: Dict) -> Dict:
        """딕셔너리의 key-value 쌍 중 value가 None인 쌍을 제거합니다.

        카카오톡 서버로 반환 시 None인 값을 제외하고 반환하기 위해 사용합니다.

        Args:
            base (dict): None인 값을 제거할 딕셔너리

        Returns:
            dict: None인 값을 제거한 딕셔너리

        Examples:
            >>> self.remove_none_item(
            ... {'key1': 'value1',
            ... 'key2': None,
            ... 'key3': 'value3'}
            ... )
            {'key1': 'value1', 'key3': 'value3'}
        """
        out = {}
        for key, value in base.items():
            if value is not None:
                out[key] = value
        return out

    @abstractmethod
    def render(self) -> Union[Dict, List]:
        """객체를 카카오톡 응답 형식에 알맞게 dict로 변환합니다.

        변환된 dict는 각 객체가 타깃으로 하는 카카오톡 응답 형식의 상세 필드를 key로 가집니다.
        TextCard 객체의 경우 다음과 같은 형식으로 변환됩니다:

        Examples:
            >>> self.render()
            {
                'title': '제목',
                'description': '설명',
                'buttons': [
                    {
                        'action': 'webLink',
                        'label': '링크',
                        'webLinkUrl': 'https://www.example.com',
                    },
                ],
            }

        Returns:
            dict: 변환된 dict 객체.
        """

    @abstractmethod
    def validate(self):
        """카카오톡 응답 규칙에 알맞은지 객체를 검증합니다.

        응답 규칙에 맞지 않을 경우 예외를 발생시키도록 구현해야 합니다.
        """


class ParentPayload(ABC, metaclass=ABCMeta):
    """Payload 객체의 부모 클래스입니다.

    스킬 실행시 봇 시스템이 스킬 서버에게 전달하는 정보를 객체화 하기 위한 Payload 클래스들은 이 클래스를 상속받아 구현됩니다.

    Abstract Methods:
        from_dict: 딕셔너리를 객체로 변환하는 메서드

    단독으로 사용 가능한 자식 클래스:
        - Payload
        - ValidationPayload
        - Context
    """

    @classmethod
    @abstractmethod
    def from_dict(cls, data: Dict):
        """딕셔너리를 받아서 객체로 변환하는 메서드입니다.

        Args:
            data (dict): 딕셔너리
        """

    @classmethod
    def from_json(cls, data: str):
        """JSON 문자열을 객체로 변환하는 메서드입니다.

        from_dict 메서드를 호출하여 JSON 문자열을 객체로 변환합니다.

        Args:
            data (str): JSON 문자열
        """
        return cls.from_dict(json.loads(data))


class SkillTemplate(BaseModel, metaclass=ABCMeta):
    """카카오톡 SkillTemplate 출력요소를 객체로 생성하는 추상 클래스입니다.

    Abstract Methods:
        - render: 객체를 카카오톡 응답 형식에 알맞게 dict로 변환합니다.
        - validate: 카카오톡 응답 규칙에 맞는지 객체를 검증합니다.

    Raises:
        NotImplementedError: render 또는 validate 메서드가 구현되지 않았을 때
    """

    @property
    def is_empty(self) -> bool:
        """응답 객체가 비어있는지 여부를 반환합니다.

        self.render() 메서드를 호출하여 응답 객체를 생성한 후,
        응답 객체가 비어있는지 여부를 반환합니다.

        Returns:
            bool: 응답 객체가 비어있으면 True, 아니면 False
        """
        return not bool(self.render())



"""카카오톡 출력 요소의 객체를 생성하는 클래스들을 정의한 모듈입니다.

주로 스킬 응답들이 공통으로 사용되는 속성을 객체화 하는 추상 클래스입니다.
Common이 아닌 다른 SkillTemplate의 출력 요소의 구성 요소로 사용됩니다.
ListItem은 ListCard의 items에 들어가는 Item을 나타내는 클래스로써
ListCard 에서만 사용됩니다.

classes:
    - Common: 카카오톡 출력 요소의 공통 속성 객체를 의미하는 추상 클래스입니다.
    - Action: 카카오톡 출력 요소의 버튼의 동작을 나타내는 열거형 클래스입니다.
    - QuickReply: 카카오톡 출력 요소 QuickReply의 객체를 생성하는 클래스입니다.
    - Link: 카카오톡 출력 요소 Link의 객체를 생성하는 클래스입니다.
    - Thumbnail: 카카오톡 출력 요소 Thumbnail의 객체를 생성하는 클래스입니다.
    - Profile: 카카오톡 출력 요소 Profile의 객체를 생성하는 클래스입니다.
    - Button: 카카오톡 출력 요소 Button의 객체를 생성하는 클래스입니다.
    - ListItem: 카카오톡 출력 요소 ListCard의 items에 들어가는 Item을 나타내는 클래스입니다.
"""

from abc import ABCMeta
from typing import Optional, Union, List, Dict


from ...customerror import InvalidLinkError
from ...validation import validate_str, validate_type
from ..interaction import ActionEnum, Interaction
from ..base import ParentComponent

__all__ = [
    "Link",
    "Thumbnail",
    "Profile",
    "ListItem",
    "Button",
]


class Common(ParentComponent, metaclass=ABCMeta):
    """스킬 응답들이 공통으로 사용되는 속성을 객체화 하는 추상 클래스입니다.

    명시적으로 스킬 응답의 공통으로 사용되는 속성을 객체화 하기 위해 사용됩니다.
    (ex. Thumbnail, Button 등)

    Common이 아닌 다른 SkillTemplate의 출력 요소의 구성 요소로 사용됩니다.
    (ex. Thumbnail은 BasicCard, CommerceCard의 속성으로 사용됩니다.)
    """


class Link(Common):
    """카카오톡 출력 요소 Link의 객체를 생성하는 클래스입니다.

    Link는 버튼이나 썸네일 등에서 사용되는 링크를 나타냅니다.
    Link 객체는 웹, PC, 모바일 링크를 가질 수 있습니다.
    web이 가장 우선적으로 실행되며, web이 없는 경우에 플랫폼에 따라 PC 또는 모바일 링크가 실행됩니다.

    ListItem, ItemThumbnail, Thumbnail, Button의 속성으로 사용됩니다.

    Args:
        web (Optional[str]): 웹 링크
        pc (Optional[str]): PC 링크
        mobile (Optional[str]): 모바일 링크

    Examples:
        >>> link = Link(web="https://www.example.com")
        >>> link.render()
        {'web': 'https://www.example.com'}
    """

    def __init__(
        self,
        web: Optional[str] = None,
        pc: Optional[str] = None,
        mobile: Optional[str] = None,
    ):
        """Link 객체의 생성자 메서드입니다.

        Args:
            web (Optional[str], optional): 웹 링크. Defaults to None.
            pc (Optional[str], optional): PC 링크. Defaults to None.
            mobile (Optional[str], optional): 모바일 링크. Defaults to None.
        """
        self.web = web
        self.pc = pc
        self.mobile = mobile

    def validate(self):
        """Link 객체의 유효성을 검사합니다.

        Link는 최소 하나의 링크를 가져야 합니다.
        self.web, self.pc, self.mobile이 모두 None인 경우 InvalidLinkError를 발생시킵니다.

        Raises:
            InvalidLinkError: 링크가 하나도 없는 경우
        """
        if self.web is None and self.pc is None and self.mobile is None:
            raise InvalidLinkError("Link는 최소 하나의 링크를 가져야 합니다.")
        validate_str(self.web, self.pc, self.mobile)

    def render(self) -> Dict:
        """Link 객체를 카카오톡 응답 형식에 맞게 딕셔너리로 변환합니다.

        response에 web, pc, mobile 링크를 저장합니다.
        이후 None인 링크는 제거후 반환합니다.

        Returns:
            dict: 카카오톡 응답 형식에 맞게 변환된 Link 딕셔너리
        """
        self.validate()
        response = {
            "web": self.web,
            "pc": self.pc,
            "mobile": self.mobile,
        }
        return self.remove_none_item(response)


class Thumbnail(Common):
    """카카오톡 출력 요소 Thumbnail의 객체를 생성하는 클래스입니다.

    Thumbnail은 썸네일 이미지를 나타냅니다.
    썸네일 이미지는 이미지 URL을 가지며, 링크를 가질 수 있습니다.
    fixedRatio는 이미지의 비율을 고정할지 여부를 나타냅니다.

    BasicCard, CommerceCard 객체의 속성으로 사용됩니다.

    Attributes:
        image_url (str): 썸네일 이미지 URL
        link (Link): 썸네일 이미지 클릭 시 이동할 링크
        fixed_ratio (bool): 이미지의 비율을 고정할지 여부
    """

    def __init__(
        self, image_url: str, link: Optional[Link] = None, fixed_ratio: bool = False
    ):
        """Thumbnail 클래스의 생성자 메서드입니다.

        Args:
            image_url (str): 썸네일 이미지 URL
            link (Link): 썸네일 이미지 클릭 시 이동할 링크, 기본값은 None
            fixed_ratio (bool): 이미지의 비율을 고정할지 여부, 기본값은 False
        """
        self.image_url = image_url
        self.link = link
        self.fixed_ratio = fixed_ratio

    def validate(self):
        """Thumbnail 객체의 유효성을 검사합니다.

        Raises:
            InvalidTypeError: image_url이 문자열이 아닌 경우
            InvalidTypeError: link가 Link 객체가 아닌 경우
            InvalidTypeError: fixed_ratio가 bool이 아닌 경우
        """
        validate_str(self.image_url)
        validate_type(Link, self.link, disallow_none=False)
        validate_type(bool, self.fixed_ratio)

    def render(self) -> Dict:
        """Thumbnail 객체를 카카오톡 응답 형식에 맞게 딕셔너리로 변환합니다.

        response에 image_url, fixed_ratio, link를 저장합니다.
        이후 None인 속성은 제거후 반환합니다.

        Returns:
            dict: 카카오톡 응답 형식에 맞게 변환된 Thumbnail 딕셔너리
        """
        self.validate()
        response = {
            "imageUrl": self.image_url,
            "fixedRatio": self.fixed_ratio if self.fixed_ratio else None,
            "link": self.link.render() if self.link is not None else None,
        }

        return self.remove_none_item(response)


class Profile(Common):
    """카카오톡 출력 요소 Profile의 객체를 생성하는 클래스입니다.

    Profile은 사용자의 프로필 정보를 나타냅니다.
    이미지 URL은 선택적으로 사용할 수 있습니다.
    이미지 사이즈는 180px X 180px이 권장됩니다.

    CommerceCard, ImageCard 객체의 속성으로 사용됩니다.

    Attributes:
        nickname (str): 사용자의 닉네임
        image_url (Optional[str]): 사용자의 프로필 이미지 URL, 기본값은 None
    """

    def __init__(self, nickname: str, image_url: Optional[str] = None):
        """Profile 클래스의 생성자 메서드입니다.

        Args:
            nickname (str): 사용자의 닉네임
            image_url (Optional[str]): 사용자의 프로필 이미지 URL, 기본값은 None
        """
        super().__init__()
        self.nickname = nickname
        self.image_url = image_url

    def validate(self):
        """Profile 객체의 유효성을 검사합니다.

        Raises:
            InvalidTypeError: nickname이 문자열이 아닌 경우
            InvalidTypeError: image_url이 문자열이 아닌 경우
        """
        validate_str(self.nickname, self.image_url)

    def render(self):
        """Profile 객체를 카카오톡 응답 형식에 맞게 딕셔너리로 변환합니다.

        nickname과 image_url을 response에 저장합니다.
        이후 None인 속성은 제거후 반환합니다.

        Returns:
            dict: 카카오톡 응답 형식에 맞게 변환된 Profile 딕셔너리
        """
        response = {
            "nickname": self.nickname,
            "imageUrl": self.image_url,
        }
        return self.remove_none_item(response)


class Button(Interaction, Common):
    """카카오톡 출력 요소 Button의 객체를 생성하는 클래스입니다.

    Button은 사용자가 챗봇에게 빠르게 응답할 수 있도록 도와주는 버튼입니다.
    TextCardComponent, BasicCardComponent, CommerceCardComponent,
    ListCardComponent, ItemCardComponent 객체의 속성으로 사용됩니다.


    Attributes:
        label (str): 버튼의 텍스트
        action (Action): 버튼의 동작
        web_link_url (str): 웹 링크
        message_text (str): 메시지
        phone_number (str): 전화번호
        block_id (str): 블록 ID
        extra (dict): 스킬 서버에 추가로 전달할 데이터

    Raises:
        ValueError: text가 문자열이 아닌 경우

    Examples:
        >>> button = Button(
        ...     label="버튼 1",
        ...     action="message",
        ...     message_text="버튼 1 클릭"
        ... )
        >>> button.render()
        {'label': '버튼 1', 'action': 'message', 'messageText': '버튼 1 클릭'}
    """

    available_action_enums: List[ActionEnum] = list(ActionEnum)

    def __init__(
        self,
        label: str,
        action: Union[str, ActionEnum],
        web_link_url: Optional[str] = None,
        message_text: Optional[str] = None,
        phone_number: Optional[str] = None,
        block_id: Optional[str] = None,
        extra: Optional[Dict] = None,
    ):
        """Button 클래스의 생성자 메서드입니다.

        Args:
            label (str): 버튼에 적히는 문구입니다.
            action (Union[str, Action]): 버튼 클릭시 수행될 작업입니다.
                                    (webLink, message, phone,
                                    block, share, operator)
            web_link_url (Optional[str]): 웹 브라우저를 열고 이동할 주소입니다.
                                            (action이 webLink일 경우 필수)
            message_text (Optional[str]): action이 message인 경우 사용자의 발화로
                                            messageText를 내보냅니다. (이 경우 필수)
                                        action이 block인 경우 블록 연결시
                                            사용자의 발화로 노출됩니다. (이 경우 필수)
            phone_number (Optional[str]): 전화번호 (action이 phone일 경우 필수)
            block_id (Optional[str]): 호출할 block_id. (action이 block일 경우 필수)
            extra (Optional[dict]): 스킬 서버에 추가로 전달할 데이터
        """
        assert action is not None, "action은 str 또는 ActionEnum이어야 합니다."
        Interaction.__init__(self, action, extra)
        self.label = label
        self.web_link_url = web_link_url
        self.message_text = message_text
        self.phone_number = phone_number
        self.block_id = block_id

    def validate(self):
        """Button 객체의 유효성을 검사합니다.

        Raises:
            InvalidTypeError: label이 문자열이 아닌 경우
            InvalidTypeError: web_link_url이 문자열이 아닌 경우
            InvalidTypeError: message_text가 문자열이 아닌 경우
            InvalidTypeError: phone_number가 문자열이 아닌 경우
            InvalidTypeError: block_id가 문자열이 아닌 경우
        """
        assert (
            self.action in self.available_action_enums
        ), f"action은 {self.available_action_enums} 중 하나여야 합니다."
        validate_str(
            self.label,
            self.web_link_url,
            self.message_text,
            self.phone_number,
            self.block_id,
        )
        super().validate()

    def render(self) -> Dict:
        """Button 객체를 카카오톡 응답 형식에 맞게 딕셔너리로 변환합니다.

        Examples:
            >>> self.render()
            {
                "label": "버튼 1",
                "action": "message",
                "messageText": "버튼 1 클릭"
            }

        Returns:
            dict: 카카오톡 응답 형식에 맞게 변환된 Button 딕셔너리
        """
        self.validate()
        response = {
            "label": self.label,
        }
        response.update(Interaction.render(self))
        return response


class ListItem(Common, Interaction):
    """카카오톡 출력 요소 ListCard의 items에 들어가는 Item을 나타내는 클래스입니다.

    ListItem은 리스트 형태의 정보를 나타냅니다.
    ListItem은 title, description, imageUrl, link, action, block_id,
    message_text, extra를 가질 수 있습니다.

    ListCard 객체의 속성으로 사용됩니다.

    Class Attributes:
        available_action_enums (list[ActionEnum]): 사용 가능한 Action 열거형의 리스트

    Attributes:
        title (str): header에 들어가는 경우, listCard의 제목, items에 들어가는 경우, 해당 항목의 제목
        description (str, optional): items에 들어가는 경우, 해당 항목의 설명
        image_url (str, optional): items에 들어가는 경우, 해당 항목의 우측 안내 사진
        link (Link, optional): 리스트 아이템 클릭 시 동작할 링크
        action (Union[str, Action], optional): 리스트 아이템 클릭시 수행될 작업(block 또는 message)
        block_id (str, optional): action이 block인 경우 block_id를 갖는 블록을 호출
        message_text (str, optional): action이 message인 경우 리스트 아이템 클릭 시 전달할 메시지
        extra (dict, optional): 블록 호출시, 스킬 서버에 추가적으로 제공하는 정보
    """

    available_action_enums = [ActionEnum.BLOCK, ActionEnum.MESSAGE]

    def __init__(
        self,
        title: str,
        description: Optional[str] = None,
        image_url: Optional[str] = None,
        link: Optional[Link] = None,
        action: Optional[Union[str, ActionEnum]] = None,
        block_id: Optional[str] = None,
        message_text: Optional[str] = None,
        extra: Optional[Dict] = None,
    ):
        """ListItem 클래스의 생성자 메서드입니다.

        Args:
            title (str): header이면 listCard, items이면 해당 항목의 제목
            description (str, optional): items에 들어가는 경우, 해당 항목의 설명
            image_url (str, optional): items에 들어가는 경우, 해당 항목의 우측 안내 사진
            link (Link, optional): 리스트 아이템 클릭 시 동작할 링크
            action (Union[str, Action], optional): 아이템 클릭시 수행될 작업(block 또는 message)
            block_id (str, optional): action이 block인 경우 block_id를 갖는 블록을 호출
            message_text (str, optional): message인 경우 리스트 아이템 클릭 시 전달할 메시지
            extra (dict, optional): 블록 호출시, 스킬 서버에 추가적으로 제공하는 정보
        """
        if action is not None:
            Interaction.__init__(self, action=action, extra=extra)
        else:
            self.action = None

        self.title = title
        self.description = description
        self.image_url = image_url
        self.link = link
        self.block_id = block_id
        self.message_text = message_text

    def validate(self):
        """ListItem 객체의 유효성을 검사합니다.

        Raises:
            InvalidTypeError: title이 문자열이 아니거나 None인 경우
            InvalidTypeError: description이 문자열이 아닌 경우
            InvalidTypeError: image_url이 문자열이 아닌 경우
            InvalidTypeError: link가 Link 객체가 아닌 경우
        """
        if self.action:
            Interaction.validate(self)
        validate_str(self.title, disallow_none=True)
        validate_str(self.description, self.image_url)
        validate_type(Link, self.link)

    def render(self):
        """ListItem 객체를 카카오톡 응답 형식에 맞게 딕셔너리로 변환합니다.

        title, description, imageUrl, link, action,
        block_id, message_text, extra를 response에 저장합니다.
        만약 action이 존재할 경우 Interaction.render()를 통해 action을 추가합니다.
        이후 None인 속성은 제거후 반환합니다.

        Returns:
            dict: 카카오톡 응답 형식에 맞게 변환된 ListItem 딕셔너리
        """
        self.validate()
        response = {
            "title": self.title,
            "description": self.description,
            "imageUrl": self.image_url,
            "link": self.link.render() if self.link else None,
        }
        if self.action:
            action_response = Interaction.render(self)
            response.update(action_response)
        return self.remove_none_item(response)
